/**
	*****************************************************************************
	* @file			azq_i2c.c
	* @ingroup	azq_i2c
	* @brief 		azq_i2c module C file.
	* @note			Any functions or variables which are meant to be used outside of
	*						this file are preceeded with 'azq_' in order to identify them when
	*						used in other files.
	*
	*****************************************************************************/

/*** Includes ***/
#include "azq_dev.h"


/*** Defines ***/

/*** Private Global Variables ***/
AZQ_Device_t *_azqPtr;

/* callback function*/

/** @brief    An array which can be used by functions for temporary storage.
  * @details  This array provides a memory area wherein functions which require
  *           temporary storage for variable length arrays can store data.
  * @note     The purpose of this array is to eliminate the use of malloc().
  * @warning  Currently this array reserved 32 bytes. This is mainly so that the
  *           counts and LTA values of all the IQS269 channels can be read at
  *           once. Change this amount to suit your application if required.
  */


/*** Private Function Declarations ***/


/*** Functions ***/

/** @brief			Initializes the software I2C bus and the ready functionality for
	*							the specified AZQ device.
  * @details    After calling this init function, the active I2C bus and AZQ
  *             device will be that of the one passed.
																																					 
																									
	*	@param[in]	azqDevice		An AZQ_Device_t type which holds the required
  *                         information for the device to be initialised.
  * @note       If using multiple devices, the user is responSible for making
  *             sure the correct device is selected by using the
  *             azq_setActiveDevice() function with the required AZQ_Device_t
  *             struct and that the correct I2C bus is selected using the
  *             i2c_setActiveBus() function with the correct I2C_SoftwareBus_t
  *             struct.
	*/
AZQ_Status_e azq_init(AZQ_Device_t *azqDevice)
{

	// Assign _azqPtr to the passed pointer.
	//_azqPtr = azqDevice;
	

	// Enable the Port clock of the ready pin.
	Enable_GPIO_Clock(azqDevice->readyPort); // todo add check for invalid port. at the moment all ports activated for default

	List_init((List_t *)(&azqDevice->i2c_queue), sizeof(I2C_transaction_s),5);


	// Configure the ready pin as a Floating Input.
	gpio_init(azqDevice->readyPort, azqDevice->readyPin, LL_GPIO_MODE_INPUT, FUNCTION_NOT_USED, FUNCTION_NOT_USED, LL_GPIO_PULL_NO, azqDevice->i2cBus.GPIOspeed);

	// Initialise the I2C bus.
	init_i2c(&azqDevice->i2cBus);

	// Call function to set-up interrupt
	azq_initReadyInterrupt(azqDevice, azqDevice->IR_priority, azqDevice->Rising_IR, azqDevice->Falling_IR);

	// call function to enable interrupt
	azq_enableReadyInterrupt(azqDevice);






	return AZQ_STATUS_SUCCESS;
}


/** @brief    Enables the interrupt on the ready pin for the selected device.
  * @note     The device should be initialised with the azq_init() function
  *           before calling this function.
  * @warning  The user must create a function to handle the interrupt and make
  *           sure the interrupt vector table points to this function for the
  *           interrupt in question.
  */
void azq_initReadyInterrupt(AZQ_Device_t *azqDevice, uint32_t priority, bool rising, bool falling)
{
	//Configure EXTI for AZQ device. todo confirm whether readyPin parameter translates directly to Line parameter.
	Configure_EXTI(azqDevice->readyPort, azqDevice->readyPin, priority, rising, falling, azqDevice->_ready_callback);

}



/** @brief    Enables the interrupt on the ready pin for the selected device.
  */
void azq_enableReadyInterrupt(AZQ_Device_t *azqDevice)
{
		LL_EXTI_EnableIT_0_31(azqDevice->readyPin);
}

/** @brief    Disables the interrupt on the ready pin for the selected device.
  */
void azq_disableReadyInterrupt(AZQ_Device_t *azqDevice)
{
		LL_EXTI_DisableIT_0_31(azqDevice->readyPin);
}

/*** Functions for Initiating Communication. ***/

/** @brief	Returns the status of the ready pin and thus the ready line.
 *  @retval	AZQ_READY_HIGH if the logic level on the ready line is high.
 *	@retval	AZQ_READY_LOW if the logic level on the ready line is low.
 */


AZQ_ReadyState_e azq_readyStatus(AZQ_Device_t *azqDevice)
{
	if(!LL_GPIO_IsInputPinSet(azqDevice->readyPort, azqDevice->readyPin))
	{
		return AZQ_READY_LOW;
	}
	else
	{
		return AZQ_READY_HIGH;
	}
}

/**
 * @brief  queue transaction is function that will be called from upper layer. This will add the transaction to the I2C condition queue
 * with correct parameters.
 * @param I2C peripheral, device addresss, register, buffer address, amount of bytes, type of transaction (READ/WRITE)
 * @retval True if function was successfull, else false.
 */


//bool queue_transaction(AZQ_Device_t *azqDevice, uint8_t reg, uint8_t *buffer, uint8_t num_words, I2C_Read_Write_e type)
//{
//	I2C_transaction_s temp_trans;
//
//	temp_trans.i2cPeripheral = azqDevice->i2cBus.i2cPeripheral;
//	temp_trans.addr = azqDevice->i2cAddress;
//	temp_trans.reg = reg;
//	temp_trans.buffer = buffer;
//	temp_trans.num_bytes= 2*num_words;
//	temp_trans.trans_type = type;
//	temp_trans.trans_last = false;
//
//
//	List_insertStart(&azqDevice->i2c_queue, 0x01, &temp_trans);
//
//	return true;
//}

