/**
	******************************************************************************
	*	@addtogroup	stm32l1_hardware_i2c	STM32L1 Hardware I2C Master
	*	@brief		  Provides hardware I2C functionality and commonly used I2C
	*						  transactions to facilitate communication between STM32
	*						  microcontrollers and Azoteq devices.
	*	@details
  *
	******************************************************************************
	*	@defgroup	azq_i2c azq_i2c
	*	@{
	*	@ingroup	stm32_hardware_i2c
  * @brief 		Provides commonly used software I2C transactions for Azoteq
  *           devices.
	* @details	The purpose of this library is to provide basic I2C operations 
	*						which work with azoteq devices. These operations can then be used
	*						to build more specific device functions. Multiple devices must be
	*						created using the AZQ_Device_t type and then the active device can
	*						be selected using the setActiveDevice() function.
	*
  ******************************************************************************
  * @file			azq_i2c.h
  * @brief 		azq_i2c module header file.
	* @note			Any functions or variables which are meant to be used outside of
	*						this file are preceeded with 'azq_' in order to identify them when
	*						used in other files.
  *
	*****************************************************************************/
#ifndef __AZQ_I2C_H
#define __AZQ_I2C_H

/*** Icludes ***/
#include "i2c.h"
#include "AZQ_util.h"
//#include "main.h"

/*** Defines ***/

/** @name   AZQ Enumerations
  * @brief
  * @{
  */

/**	@brief    An enumerator which allows selection of the End Condition for a
  * 					transaction.
  * @details  Allows the user to specify whether the transaction must end in a
  *           Stop command or a Restart command.
  */
//typedef enum {
//	AZQ_END_RESTART = I2C_END_RESTART,
//	AZQ_END_STOP		= I2C_END_STOP
//} AZQ_EndCondition_e;

/**	@brief 	  An enumerator indicating the various transaction status
  *           conditions.
  * @details  See the "retval" documentation for each function to see why
  *           certain status conditions are returned.
  */
typedef enum {
	AZQ_STATUS_SUCCESS				= I2C_STATUS_SUCCESS,
	AZQ_STATUS_NACK 				= I2C_STATUS_NACK,
	AZQ_STATUS_I2C_FAULT			= 0x02,
	AZQ_STATUS_FAIL					= 0x04,
	AZQ_STATUS_TIMEOUT				= 0x08
} AZQ_Status_e;

/** @brief    An enumerator which indicates which states the ready line can be
  *           in.
  */
typedef enum {
	AZQ_READY_LOW   = 0x00,
  AZQ_READY_HIGH  = 0x01
} AZQ_ReadyState_e;

typedef enum {

	INT_NO_INTERRUPT 				= (uint8_t) 0x00,
	INT_RISING_INTERRUPT,
	INT_FALLING_INTERRUPT
} interrupt_e;

/** @} End group AZQ Enumerations */

/*** Types ***/

/** @name   AZQ Types
  * @brief
  * @{
  */

/**	@brief		A struct which holds the unique information for an AZQ device.
	*	@details	Switching between devices can be done by calling the
	*						azq_setActiveDevice() method.
	*	@see			azq_setActiveDevice()
	*/

#pragma pack(1)
typedef struct{
	I2C_HardwareBus_t i2cBus;
	uint8_t i2cAddress;
	GPIO_TypeDef *readyPort;
	uint32_t readyPin;
	uint16_t azqTimeout_ms;
	void	(*_ready_callback)(void);
	uint8_t Interrupt_Type;
	uint32_t IR_priority;
	uint8_t Falling_IR;
	uint8_t Rising_IR;
	List_t i2c_queue;
} AZQ_Device_t;
#pragma pack(4)


/** @} End group AZQ Types */

/*** Function Declarations ***/
// Initialisation Functions

AZQ_Status_e azq_init(AZQ_Device_t *azqDevice);

void azq_initReadyInterrupt(AZQ_Device_t *azqDevice, uint32_t priority, bool rising, bool falling);

void azq_enableReadyInterrupt(AZQ_Device_t *azqDevice);

void azq_disableReadyInterrupt(AZQ_Device_t *azqDevice);

// Functions for Initiating Communication.

AZQ_ReadyState_e azq_readyStatus(AZQ_Device_t *azqDevice);

bool queue_transaction(AZQ_Device_t *azqDevice, uint8_t reg, uint8_t *buffer, uint8_t num_words, I2C_Read_Write_e type);



#endif	//	__AZQ_I2C_H

/**	@}
	*	End group azq_i2c
	*/
	
