/**
	******************************************************************************
	*	@addtogroup	azq_drivers	STM32 AZQ Drivers
	*	@brief		  Provides drivers for azoteq devices based on the azq_dev module.
	******************************************************************************
  *	@defgroup	  azq_drivers_iqs269  azq_iqs269
  * @{
  * @ingroup	  azq_drivers
	* @brief 		  IQS269 driver for STM32 devices.
	* @details	  Provides functions for interacting with the Azoteq IQS269 device.
  *             This driver is built on the azq_i2c module.
  *
	******************************************************************************
  * @file			  iqs269.h
	*	@brief      IQS269 driver module header fle.
  * @note			  Any functions or variables which are meant to be used outside of
	*						  this file are preceeded with 'iqs269_' in order to identify them
	*						  when used in other files.
	*
	*****************************************************************************/

/* Prevent Recursive inclusion. */
#ifndef __IQS269_H
#define __IQS269_H

/*** Includes ***/
#include "azq_dev.h"
#include "i2c.h"
#include "iqs269_addresses.h"
#include "iqs269_typedefs.h"

#define IQS269_WAIT_FOR_RESET_DELAY				250
#define	IQS269_TIMEOUT							60000

//#define IQS269_8BUTTON_DEMO



/* Exported types ------------------------------------------------------------*/
/**
* @brief  IQS269 State Enumeration.
*/
typedef enum {
	IQS269_STATE_NONE = (uint8_t) 0x00,			// 0x00
	IQS269_STATE_INIT,							// 0x01
	IQS269_STATE_IDLE,							 // 0x02 - wait for data ready
	IQS269_STATE_WAIT,
	IQS269_STATE_DATA_READY,					// 0x03 - check for data ready and start the data read						// 0x04 - retrieve the data and set to done					// 0x05 - error checking state
} IQS269_State_e;


/**
* @brief  IQS269 update state
*/
typedef enum {
	IQS269_UPDATE_NONE = (uint8_t) 0x00,			// 0x00
	IQS269_UPDATE_INIT,
	IQS269_UPDATE_BUSY,
	IQS269_UPDATE_DONE,
	IQS269_UPDATE_ERROR
} IQS269_update_e;

/**
* @brief  IQS269 Init Enumeration.
*/
typedef enum {
	IQS269_INIT_NONE = (uint8_t) 0x00,			// 0x00
	IQS269_INIT_INTERFACE,
	IQS269_INIT_CHIP_RESET,
	IQS269_INIT_READ_RESET,
	IQS269_INIT_CHECK_RESET,
	IQS269_INIT_ACK_RESET,
	IQS269_INIT_PMU_SETTINGS,
	IQS269_INIT_ATI,
	IQS269_INIT_DONE
} IQS269_init_e;


/* Product information starting at 0x00 */
#pragma pack(1)
typedef union{
		struct{
			union
			{
				struct
				{
					uint8_t prod_num : 8;
				};
				uint8_t iqs269_prod_num;
			};

			union
			{
				struct
				{
					uint8_t soft_ver : 8;
				};
				uint8_t iqs269_soft_ver;
			};

			union
			{
				struct
				{
					uint8_t hard_num : 8;
				};
				uint8_t iqs269_hard_num;
			};

			union
			{
				struct
				{
					uint8_t prod_res : 8;
				};
				uint8_t iqs269_prod_res;
			};
		};
		uint8_t buffer[4];
} IQS269_PROD_INFO;
#pragma pack(4)

/* SYSTEM FLAGS (Starting address 0x02 */
#pragma pack(1)
typedef union {
		struct {

			// System Flags
			union
			{
				struct
				{
					uint8_t ulp_update : 1;  // 0
					uint8_t global_event : 1;
					uint8_t in_ati : 1;
					uint8_t Power_mode : 2;
					uint8_t sys_placekeeper_5 : 1;
					uint8_t sys_placekeeper_6 : 1;
					uint8_t Show_reset : 1; // 7
				};
				uint8_t iqs269_system_flags;
			};

			// Global Events
			union
			{
				struct
				{
					uint8_t prox_event : 1;  //0
					uint8_t touch_event : 1;
					uint8_t deep_touch_event : 1;
					uint8_t gesture : 1;
					uint8_t glo_evt_placekeeper_4 : 1;
					uint8_t ref_channel : 1;
					uint8_t system : 1;
					uint8_t power_mode_event : 1;  //7
				};
				uint8_t iqs269_global_ev_flags;
			};
	};
	uint8_t buffer[2];
} IQS269_SYS_GE_FLAGS;
#pragma pack(4)


/* Slider Event Flags (Starting address 0x03 */
#pragma pack(1)
typedef union {
		struct {

			// Gesture (Slider1 & Slider 0) event flags
			union
			{
				struct
				{
					uint8_t tap_0 		: 1;  // 0
					uint8_t hold_0 		: 1;
					uint8_t flick_pos_0 : 1;
					uint8_t flick_neg_0 : 1;
					uint8_t tap_1 		: 1;
					uint8_t hold_1 		: 1;
					uint8_t flick_pos_1 : 1;
					uint8_t flick_neg_1 : 1; // 7
				};
				uint8_t iqs269_gesture_events;
			};

			// Reserved
			union
			{
				struct
				{
					uint8_t sli_res : 8;  //0
				};
				uint8_t iqs269_sldier_reserved;
			};
	};
	uint8_t buffer[2];
} IQS269_GESTURE_EV;
#pragma pack(4)


/* Channel States (Starting address 0x04 */
#pragma pack(1)
typedef union {
		struct {

			// Channel Proximity State
			union
			{
				struct
				{
					uint8_t ch0_prox : 1;  // 0
					uint8_t ch1_prox : 1;
					uint8_t ch2_prox : 1;
					uint8_t ch3_prox : 1;
					uint8_t ch4_prox : 1;
					uint8_t ch5_prox : 1;
					uint8_t ch6_prox : 1;
					uint8_t ch7_prox	: 1; // 7
				};
				uint8_t iqs269_chan_prox_state;
			};

			// Channels Proximity direction state
			union
			{
				struct
				{
					uint8_t ch0_prox_di : 1;  // 0
					uint8_t ch1_prox_di : 1;
					uint8_t ch2_prox_di : 1;
					uint8_t ch3_prox_di : 1;
					uint8_t ch4_prox_di : 1;
					uint8_t ch5_prox_di : 1;
					uint8_t ch6_prox_di : 1;
					uint8_t ch7_prox_di	: 1; // 7
				};
				uint8_t iqs269_chan_prox_di_state;
			};

			// Channels Touch State
			union
			{
				struct
				{
					uint8_t ch0_touch : 1;  // 0
					uint8_t ch1_touch : 1;
					uint8_t ch2_touch : 1;
					uint8_t ch3_touch : 1;
					uint8_t ch4_touch : 1;
					uint8_t ch5_touch : 1;
					uint8_t ch6_touch : 1;
					uint8_t ch7_touch : 1; // 7
				};
				uint8_t iqs269_chan_touch_state;
			};

			// Channels Deep Touch state
			union
			{
				struct
				{
					uint8_t ch0_d_touch : 1;  // 0
					uint8_t ch1_d_touch : 1;
					uint8_t ch2_d_touch : 1;
					uint8_t ch3_d_touch : 1;
					uint8_t ch4_d_touch : 1;
					uint8_t ch5_d_touch : 1;
					uint8_t ch6_d_touch : 1;
					uint8_t ch7_d_touch	: 1; // 7
				};
				uint8_t iqs269_chan_touch_deep_state;
			};

			// Reference channels actively used
			union
			{
				struct
				{
					uint8_t ch0_ref : 1;  // 0
					uint8_t ch1_ref : 1;
					uint8_t ch2_ref : 1;
					uint8_t ch3_ref : 1;
					uint8_t ch4_ref : 1;
					uint8_t ch5_ref : 1;
					uint8_t ch6_ref : 1;
					uint8_t ch7_ref	: 1; // 7
				};
				uint8_t iqs269_ref_chan_used;
			};

			// Reserved
			union
			{
				struct
				{
					uint8_t res : 8;  // 0

				};
				uint8_t iqs269_chan_res1;
			};

			// Reserved
			union
			{
				struct
				{
					uint8_t chan_res2 : 8;  // 0

				};
				uint8_t iqs269_chan_res2;
			};

			// Reserved
			union
			{
				struct
				{
					uint8_t chan_res3 : 8;  // 0

				};
				uint8_t iqs269_chan_res3;
			};


	};
	uint8_t buffer[8];
} IQS269_CHAN_STATES;
#pragma pack(4)

/* Count, reference, delta & slider values (Starting address 0x08 */
#pragma pack(1)
typedef union {
		struct {

			// Filtered Counts Channel 0 Byte 1
			union
			{
				struct
				{
					uint8_t ch0_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch0_filt_ct_1;
			};

			// Filtered Counts Channel 0 Byte 2
			union
			{
				struct
				{
					uint8_t ch0_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch0_filt_ct_2;
			};

			// LTA Channel 0 Byte 1
			union
			{
				struct
				{
					uint8_t ch0_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch0_lta_1;
			};

			// LTA Channel 0 Byte 2
			union
			{
				struct
				{
					uint8_t ch0_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch0_lta_2;
			};

			// Filtered Counts Channel 1 Byte 1
			union
			{
				struct
				{
					uint8_t ch1_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch1_filt_ct_1;
			};

			// Filtered Counts Channel 1 Byte 2
			union
			{
				struct
				{
					uint8_t ch1_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch1_filt_ct_2;
			};


			// LTA Channel 1 Byte 1
			union
			{
				struct
				{
					uint8_t ch1_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch1_lta_1;
			};

			// LTA Channel 1 Byte 2
			union
			{
				struct
				{
					uint8_t ch1_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch1_lta_2;
			};

			// Filtered Counts Channel 2 Byte 1
			union
			{
				struct
				{
					uint8_t ch2_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch2_filt_ct_1;
			};

			// Filtered Counts Channel 2 Byte 2
			union
			{
				struct
				{
					uint8_t ch2_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch2_filt_ct_2;
			};

			// LTA Channel 2 Byte 1
			union
			{
				struct
				{
					uint8_t ch2_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch2_lta_1;
			};

			// LTA Channel 2 Byte 2
			union
			{
				struct
				{
					uint8_t ch2_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch2_lta_2;
			};

			// Filtered Counts Channel 3 Byte 1
			union
			{
				struct
				{
					uint8_t ch3_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch3_filt_ct_1;
			};

			// Filtered Counts Channel 3 Byte 2
			union
			{
				struct
				{
					uint8_t ch3_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch3_filt_ct_2;
			};

			// LTA Channel 3 Byte 1
			union
			{
				struct
				{
					uint8_t ch3_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch3_lta_1;
			};

			// LTA Channel 3 Byte 2
			union
			{
				struct
				{
					uint8_t ch3_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch3_lta_2;
			};

			// Filtered Counts Channel 4 Byte 1
			union
			{
				struct
				{
					uint8_t ch4_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch4_filt_ct_1;
			};

			// Filtered Counts Channel 4 Byte 2
			union
			{
				struct
				{
					uint8_t ch4_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch4_filt_ct_2;
			};


			// LTA Channel 4 Byte 1
			union
			{
				struct
				{
					uint8_t ch4_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch4_lta_1;
			};

			// LTA Channel 4 Byte 2
			union
			{
				struct
				{
					uint8_t ch4_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch4_lta_2;
			};

			// Filtered Counts Channel 5 Byte 1
			union
			{
				struct
				{
					uint8_t ch5_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch5_filt_ct_1;
			};

			// Filtered Counts Channel 5 Byte 2
			union
			{
				struct
				{
					uint8_t ch5_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch5_filt_ct_2;
			};

			// LTA Channel 5 Byte 1
			union
			{
				struct
				{
					uint8_t ch5_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch5_lta_1;
			};

			// LTA Channel 5 Byte 2
			union
			{
				struct
				{
					uint8_t ch5_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch5_lta_2;
			};


			// Filtered Counts Channel 6 Byte 1
			union
			{
				struct
				{
					uint8_t ch6_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch6_filt_ct_1;
			};

			// Filtered Counts Channel 6 Byte 2
			union
			{
				struct
				{
					uint8_t ch6_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch6_filt_ct_2;
			};


			// LTA Channel 6 Byte 1
			union
			{
				struct
				{
					uint8_t ch6_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch6_lta_1;
			};

			// LTA Channel 6 Byte 2
			union
			{
				struct
				{
					uint8_t ch6_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch6_lta_2;
			};

			// Filtered Counts Channel 7 Byte 1
			union
			{
				struct
				{
					uint8_t ch7_filt_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch7_filt_ct_1;
			};

			// Filtered Counts Channel 7 Byte 2
			union
			{
				struct
				{
					uint8_t ch7_filt_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch7_filt_ct_2;
			};

			// LTA Channel 7 Byte 1
			union
			{
				struct
				{
					uint8_t ch7_lta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch7_lta_1;
			};

			// LTA Channel 7 Byte 2
			union
			{
				struct
				{
					uint8_t ch7_lta_2 : 8;  //0
				};
				uint8_t iqs269_ch7_lta_2;
			};
	};
	uint8_t buffer[32];
} IQS269_FILTERED_COUNTS;
#pragma pack(4)


/* Delta counts Channel X starting at 0x18 */
#pragma pack(1)
typedef union {
		struct {

			// Delta Counts Channel 0 Byte 1
			union
			{
				struct
				{
					uint8_t ch0_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch0_dta_ct_1;
			};

			// Delta Counts Channel 0 Byte 2
			union
			{
				struct
				{
					uint8_t ch0_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch0_dta_ct_2;
			};

			// Delta Counts Channel 1 Byte 1
			union
			{
				struct
				{
					uint8_t ch1_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch1_dta_ct_1;
			};

			// Delta Counts Channel 1 Byte 2
			union
			{
				struct
				{
					uint8_t ch1_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch1_dta_ct_2;
			};

			// Delta Counts Channel 2 Byte 1
			union
			{
				struct
				{
					uint8_t ch2_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch2_dta_ct_1;
			};

			// Delta Counts Channel 2 Byte 2
			union
			{
				struct
				{
					uint8_t ch2_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch2_dta_ct_2;
			};


			// Delta Counts Channel 3 Byte 1
			union
			{
				struct
				{
					uint8_t ch3_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch3_dta_ct_1;
			};

			// Delta Counts Channel 3 Byte 2
			union
			{
				struct
				{
					uint8_t ch3_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch3_dta_ct_2;
			};

			// Delta Counts Channel 4 Byte 1
			union
			{
				struct
				{
					uint8_t ch4_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch4_dta_ct_1;
			};

			// Delta Counts Channel 4 Byte 2
			union
			{
				struct
				{
					uint8_t ch4_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch4_dta_ct_2;
			};

			// Delta Counts Channel 5 Byte 1
			union
			{
				struct
				{
					uint8_t ch5_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch5_dta_ct_1;
			};

			// Delta Counts Channel 5 Byte 2
			union
			{
				struct
				{
					uint8_t ch5_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch5_dta_ct_2;
			};

			// Delta Counts Channel 6 Byte 1
			union
			{
				struct
				{
					uint8_t ch6_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch6_dta_ct_1;
			};

			// Delta Counts Channel 6 Byte 2
			union
			{
				struct
				{
					uint8_t ch6_dta_ct_2 : 8;  //0
				};
				uint8_t iqs269_ch6_dta_ct_2;
			};

			// Delta Counts Channel 7 Byte 1
			union
			{
				struct
				{
					uint8_t ch7_dta_ct_1 		: 8;  // 0
				};
				uint8_t iqs269_ch7_dta_ct_1;
			};

			// Delta Counts Channel 7 Byte 2
			union
			{
				struct
				{
					uint8_t ch7_dta_ct_2 		: 8;  // 0
				};
				uint8_t iqs269_ch7_dta_ct_2;
			};

	};
	uint8_t buffer[16];
} IQS269_DELTA_COUNTS;
#pragma pack(4)


/* Reference Channel Delta of Channel X (CHX Weight Applied) starting at 0x20 */
#pragma pack(1)
typedef union {
		struct {

			// Ref Chan Delta Counts Channel 0 Byte 1 IQS269_MM_DCF_DTA_CH0_L
			union
			{
				struct
				{
					uint8_t ch0_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch0_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 0 Byte 2
			union
			{
				struct
				{
					uint8_t ch0_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch0_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 1 Byte 1
			union
			{
				struct
				{
					uint8_t ch1_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch1_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 1 Byte 2
			union
			{
				struct
				{
					uint8_t ch1_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch1_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 2 Byte 1
			union
			{
				struct
				{
					uint8_t ch2_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch2_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 2 Byte 2
			union
			{
				struct
				{
					uint8_t ch2_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch2_dcf_dta_2;
			};


			// Ref Chan Delta Counts Channel 3 Byte 1
			union
			{
				struct
				{
					uint8_t ch3_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch3_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 3 Byte 2
			union
			{
				struct
				{
					uint8_t ch3_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch3_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 4 Byte 1
			union
			{
				struct
				{
					uint8_t ch4_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch4_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 4 Byte 2
			union
			{
				struct
				{
					uint8_t ch4_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch4_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 5 Byte 1
			union
			{
				struct
				{
					uint8_t ch5_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch5_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 5 Byte 2
			union
			{
				struct
				{
					uint8_t ch5_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch5_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 6 Byte 1
			union
			{
				struct
				{
					uint8_t ch6_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch6_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 6 Byte 2
			union
			{
				struct
				{
					uint8_t ch6_dcf_dta_2 : 8;  //0
				};
				uint8_t iqs269_ch6_dcf_dta_2;
			};

			// Ref Chan Delta Counts Channel 7 Byte 1
			union
			{
				struct
				{
					uint8_t ch7_dcf_dta_1 		: 8;  // 0
				};
				uint8_t iqs269_ch7_dcf_dta_1;
			};

			// Ref Chan Delta Counts Channel 7 Byte 2
			union
			{
				struct
				{
					uint8_t ch7_dcf_dta_2 		: 8;  // 0
				};
				uint8_t iqs269_ch7_dcf_dta_2;
			};

	};
	uint8_t buffer[16];
} IQS269_REF_CHAN_DELTA;
#pragma pack(4)


/* Reserved starting 0x28 */
#pragma pack(1)
typedef union {
		struct{

			union
			{
				struct
				{
					uint8_t res1 : 8;
				};
				uint8_t iqs269_res1;

			};

			union
			{
				struct
				{
					uint8_t res2 : 8;
				};
				uint8_t iqs269_res2;

			};

			union
			{
				struct
				{
					uint8_t res3 : 8;
				};
				uint8_t iqs269_res3;

			};

			union
			{
				struct
				{
					uint8_t res4 : 8;
				};
				uint8_t iqs269_res4;

			};

			union
			{
				struct
				{
					uint8_t res5 : 8;
				};
				uint8_t iqs269_res5;

			};

			union
			{
				struct
				{
					uint8_t res6 : 8;
				};
				uint8_t iqs269_res6;

			};

			union
			{
				struct
				{
					uint8_t res7 : 8;
				};
				uint8_t iqs269_res7;

			};

			union
			{
				struct
				{
					uint8_t res8 : 8;
				};
				uint8_t iqs269_res8;

			};

			union
			{
				struct
				{
					uint8_t res9 : 8;
				};
				uint8_t iqs269_res9;

			};

			union
			{
				struct
				{
					uint8_t res10 : 8;
				};
				uint8_t iqs269_res10;

			};

			union
			{
				struct
				{
					uint8_t res11 : 8;
				};
				uint8_t iqs269_res11;

			};

			union
			{
				struct
				{
					uint8_t res12 : 8;
				};
				uint8_t iqs269_res12;

			};

			union
			{
				struct
				{
					uint8_t res13 : 8;
				};
				uint8_t iqs269_res13;

			};

			union
			{
				struct
				{
					uint8_t res14 : 8;
				};
				uint8_t iqs269_res14;

			};

			union
			{
				struct
				{
					uint8_t res15 : 8;
				};
				uint8_t iqs269_res15;

			};

			union
			{
				struct
				{
					uint8_t res16 : 8;
				};
				uint8_t iqs269_res16;

			};


		};
		uint8_t buffer[16];
} IQS269_RESERVED;
#pragma pack(4)

/* Slider coordinates at 0x30 */
#pragma pack(1)
typedef union{
		struct{
			union
			{
				struct
				{
					uint8_t slid0_co : 8;
				};
				uint8_t iqs269_slid0_co;
			};

			union
			{
				struct
				{
					uint8_t slid1_co : 8;
				};
				uint8_t iqs269_slid1_co;
			};
		};
		uint8_t buffer[2];
} IQS269_SLIDER_CO;
#pragma pack(4)

/* Absolute Capacitance starting at 0x31*/
#pragma pack(1)
typedef union{
		struct{
			union
			{
				struct
				{
					uint8_t abscap_base_l : 8;
				};
				uint8_t iqs269_abscap_base_l;
			};

			union
			{
				struct
				{
					uint8_t abscap_base_h : 8;
				};
				uint8_t iqs269_abscap_base_h;
			};

			union
			{
				struct
				{
					uint8_t abscap_cnt_l : 8;
				};
				uint8_t iqs269_abscap_cnt_l;
			};

			union
			{
				struct
				{
					uint8_t abscap_cnt_h : 8;
				};
				uint8_t iqs269_abscap_cnt_h;
			};

			union
			{
				struct
				{
					uint8_t abscap_calcap0_l : 8;
				};
				uint8_t iqs269_abscap_calcap0_l;
			};

			union
			{
				struct
				{
					uint8_t abscap_calcap0_h : 8;
				};
				uint8_t iqs269_abscap_calcap0_h;
			};

			union
			{
				struct
				{
					uint8_t abscap_calcap1_l : 8;
				};
				uint8_t iqs269_abscap_calcap1_l;
			};

			union
			{
				struct
				{
					uint8_t abscap_calcap1_h : 8;
				};
				uint8_t iqs269_abscap_calcap1_h;
			};
		};
		uint8_t buffer[8];
} IQS269_ABS_CAP;
#pragma pack(4)


/* HALL Calibration data starting at 0x35*/
#pragma pack(1)
typedef union{
		struct{
			union
			{
				struct
				{
					uint8_t hall_hl : 4;
					uint8_t hall_hr : 4;
				};
				uint8_t iqs269_hall_bin;
			};

			union
			{
				struct
				{
					uint8_t cal_res1 : 8;
				};
				uint8_t iqs269_cal_res1;
			};

			union
			{
				struct
				{
					uint8_t cal_res2 : 8;
				};
				uint8_t iqs269_cal_res2;
			};

			union
			{
				struct
				{
					uint8_t cal_res3 : 8;
				};
				uint8_t iqs269_cal_res3;
			};
		};
		uint8_t buffer[4];
} IQS269_HALL_CALL;
#pragma pack(4)

/* Power Mode and System Settings (Starting 0x80) */

#pragma pack(1)
typedef	union {
		struct {

			// Power Mode General Settings
			union
			{
				struct
				{
					uint8_t ULP_UpdateRate	: 3;
					uint8_t PM_Select 		: 2;
					uint8_t Auto_PM_Switch 	: 1;
					uint8_t EN_CH0_ULP 		: 1;
					uint8_t Main_Osc 		: 1;
				};
				uint8_t iqs269_pmu_general_settings;
			};

			// General settings & commands
			union
			{
				struct
				{
					uint8_t ACK_RESET 	: 1;
					uint8_t SOFT_RESET 	: 1;
					uint8_t REDO_ATI 	: 1;
					uint8_t ADV3 		: 1;
					uint8_t ADV2 		: 1;
					uint8_t Event_Mode	: 1;
					uint8_t ADV1 	  	: 1;
					uint8_t Slider_UI 	: 1;
				};
				uint8_t iqs269_i2c_general_settings;
			};

			// Active Channels
			union
			{
				struct
				{
					uint8_t CH0_EN : 1;
					uint8_t CH1_EN : 1;
					uint8_t CH2_EN : 1;
					uint8_t CH3_EN : 1;
					uint8_t CH4_EN : 1;
					uint8_t CH5_EN : 1;
					uint8_t CH6_EN : 1;
					uint8_t CH7_EN : 1;
				};
				uint8_t iqs269_sys_chb_active;
			};

			// Raw count and LTA filter settings
			union
			{
				struct
				{
					uint8_t CF_NP  : 2;
					uint8_t LTA_NP : 2;
					uint8_t CF_LP  : 2;
					uint8_t LTA_LP : 2;
				};
				uint8_t iqs269_acf_lta_filter_settings;
			};

			// Channel Reseed Enable (Enable "LTA Halt time-out")
			union
			{
				struct
				{
					uint8_t CH0_LTA_EN_RE : 1;
					uint8_t CH1_LTA_EN_RE : 1;
					uint8_t CH2_LTA_EN_RE : 1;
					uint8_t CH3_LTA_EN_RE : 1;
					uint8_t CH4_LTA_EN_RE : 1;
					uint8_t CH5_LTA_EN_RE : 1;
					uint8_t CH6_LTA_EN_RE : 1;
					uint8_t CH7_LTA_EN_RE : 1;
				};
				uint8_t iqs269_lta_chb_reseed_enabled;
			};


			// Global event mask
			union
			{
				struct
				{
					uint8_t Prox 		: 1;
					uint8_t Touch 		: 1;
					uint8_t Deep_Touch 	: 1;
					uint8_t Gesture 	: 1;
					uint8_t Reserve 	: 1;
					uint8_t Ref_Chan 	: 1;
					uint8_t System 		: 1;
					uint8_t PM_Change 	: 1;
				};
				uint8_t iqs269_uis_global_events_mask;
			};
		};
		uint8_t buffer[6];
} IQS269_POWER_SYS_SETTINGS;
#pragma pack(4)


/* Report rates and timings (Starting 0x83) */
#pragma pack(1)
typedef union {
		struct {

			// Normal Power Report Rate
			union
			{
				struct
				{
					uint8_t NP_RR 		: 8;
				};
				uint8_t iqs269_pmu_report_rate_np;
			};

			// Low Power Report Rate
			union
			{
				struct
				{
					uint8_t LP_RR 	: 8;

				};
				uint8_t iqs269_pmu_report_rate_lp;
			};

			// Ultra-low power report rate
			union
			{
				struct
				{
					uint8_t ULP_RR : 8;
				};
				uint8_t iqs269_pmu_report_rate_ulp;
			};

			// Power mode timer
			union
			{
				struct
				{
					uint8_t PMT : 8;
				};
				uint8_t iqs269_pmu_mode_timout;
			};

			// RDY time-out
			union
			{
				struct
				{
					uint8_t RDY_TIM : 8;
				};
				uint8_t iqs269_i2c_window_timeout;
			};


			// LTA Halt timeout (proximity/Touch timeout)
			union
			{
				struct
				{
					uint8_t LTA_HALT 	: 8;
				};
				uint8_t iqs269_lta_halt_timeout;
			};
		};
		uint8_t buffer[6];
} IQS269_REPORT_AND_TIMINGS;
#pragma pack(4)


/* Global Settings (Starting 0x86) */
#pragma pack(1)
typedef union {
		struct {

			// General Settings 0
			union
			{
				struct
				{
					uint8_t GPIO_T_O_CHA 		: 3;
					uint8_t Gen_res1			: 1;
					uint8_t dis_ct_filt			: 1;
					uint8_t ati_band			: 1;
					uint8_t ati_lp				: 1;
					uint8_t Gen_adv1			: 1;
				};
				uint8_t iqs269_general_settings0;
			};

			// General Settings 1
			union
			{
				struct
				{
					uint8_t glo_cal_cap 	: 1;
					uint8_t Gen_res2		: 3;
					uint8_t Gen_adv2		: 2;
					uint8_t Bi_direct		: 1;
					uint8_t Gen_res3		: 1;

				};
				uint8_t iqs269_general_settings1;
			};

			// Absolute Capacitance Settings
			union
			{
				struct
				{
					uint8_t abscap_ui_channel_selection : 3;
					uint8_t abscap_ui_enable			: 1;
					uint8_t abscap_refcap0_sel			: 2;
					uint8_t abscap_refcap1_sel			: 2;
				};
				uint8_t iqs269_abs_cap_settings;
			};

			// Reference channel and other general settings
			union
			{
				struct
				{
					uint8_t slider_filt_strength : 2;
					uint8_t Gen_res5			 : 2;
					uint8_t en_ref_chan_track	 : 1;
					uint8_t Gen_res6			 : 1;
					uint8_t Ref_chan_def_UI		 : 2;
				};
				uint8_t iqs269_ref_chan_gensetting;
			};

			// Event blocking channel enable
			union
			{
				struct
				{
					uint8_t ch0_ev_block : 1;
					uint8_t ch1_ev_block : 1;
					uint8_t ch2_ev_block : 1;
					uint8_t ch3_ev_block : 1;
					uint8_t ch4_ev_block : 1;
					uint8_t ch5_ev_block : 1;
					uint8_t ch6_ev_block : 1;
					uint8_t ch7_ev_block : 1;
				};
				uint8_t iqs269_ev_block;
			};


			// Reserved
			union
			{
				struct
				{
					uint8_t Gen_res7 	: 8;
				};
				uint8_t iqs269_gen_res7;
			};


			// Channel selection for slider 0
			union
			{
				struct
				{
					uint8_t ch0_slid0 : 1;
					uint8_t ch1_slid0 : 1;
					uint8_t ch2_slid0 : 1;
					uint8_t ch3_slid0 : 1;
					uint8_t ch4_slid0 : 1;
					uint8_t ch5_slid0 : 1;
					uint8_t ch6_slid0 : 1;
					uint8_t ch7_slid0 : 1;
				};
				uint8_t iqs269_chan_sel_slid0;
			};

			// Channel selection for slider 1
			union
			{
				struct
				{
					uint8_t ch0_slid1 : 1;
					uint8_t ch1_slid1 : 1;
					uint8_t ch2_slid1 : 1;
					uint8_t ch3_slid1 : 1;
					uint8_t ch4_slid1 : 1;
					uint8_t ch5_slid1 : 1;
					uint8_t ch6_slid1 : 1;
					uint8_t ch7_slid1 : 1;
				};
				uint8_t iqs269_chan_sel_slid1;
			};

			// TAP timeout on slider
			union
			{
				struct
				{
					uint8_t tap_timout_slid : 8;

				};
				uint8_t iqs269_tap_timout_slid;
			};

			// Swipe gesture timeout on slider
			union
			{
				struct
				{
					uint8_t swipe_timout_slid : 8;

				};
				uint8_t iqs269_swipe_timout_slid;
			};

			// Swipe gesture threshold on slider
			union
			{
				struct
				{
					uint8_t swipe_thres_slid : 8;

				};
				uint8_t iqs269_swipe_thres_slid;
			};

			// CMD: Reseed enable OR ATI CHannel selection if "Redo ATI" bit is set
			union
			{
				struct
				{
					uint8_t ch0_re_en : 1;
					uint8_t ch1_re_en : 1;
					uint8_t ch2_re_en : 1;
					uint8_t ch3_re_en : 1;
					uint8_t ch4_re_en : 1;
					uint8_t ch5_re_en : 1;
					uint8_t ch6_re_en : 1;
					uint8_t ch7_re_en : 1;

				};
				uint8_t iqs269_chb_reseed_en;
			};
		};
		uint8_t buffer[12];
} IQS269_GLOBAL_SETTINGS;
#pragma pack(4)


/* Channel 0 Settings (Starting 0x8C) */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch0_crx0_en 		: 1; //0
					uint8_t ch0_crx1_en 		: 1;
					uint8_t ch0_crx2_en 		: 1;
					uint8_t ch0_crx3_en 		: 1;
					uint8_t ch0_crx4_en 		: 1;
					uint8_t ch0_crx5_en 		: 1;
					uint8_t ch0_crx6_en 		: 1;
					uint8_t ch0_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch0;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch0_ctx0_en 		: 1; //0
					uint8_t ch0_ctx1_en 		: 1; //0
					uint8_t ch0_ctx2_en 		: 1; //0
					uint8_t ch0_ctx3_en 		: 1; //0
					uint8_t ch0_ctx4_en 		: 1; //0
					uint8_t ch0_ctx5_en 		: 1; //0
					uint8_t ch0_ctx6_en 		: 1; //0
					uint8_t ch0_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch0;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch0_ati_mode 	  : 2;
					uint8_t ch0_proxctrl_res1 : 2;
					uint8_t ch0_int_cap_size  : 1;
					uint8_t ch0_proxctrl_adv1 : 1;
					uint8_t ch0_proxctrl_adv2 : 1;
					uint8_t ch0_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch0;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch0_sensor_mode 	  : 4;
					uint8_t ch0_proxcfg_res1 	  : 1;
					uint8_t ch0_proj_bias_cur     : 2;
					uint8_t ch0_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch0;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch0_stat_fe			   : 1;
					uint8_t ch0_sense_freq		   : 2;
					uint8_t ch0_proxcfg1_res1	   : 1;
					uint8_t ch0_proxcfg1_res2	   : 1;
					uint8_t ch0_en_int_cap		   : 1;
					uint8_t ch0_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch0;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch0_ati_target	: 6;
					uint8_t ch0_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch0;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch0_fine_op		: 4;
					uint8_t ch0_coarse_op	: 2;
					uint8_t	ch0_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch0;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch0_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch0;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch0_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch0;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch0_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch0;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch0_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch0;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch0_touch_hyst 	: 4;
					uint8_t ch0_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch0;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch0_ref_ch0 	: 1;
					uint8_t ch0_ref_ch1 	: 1;
					uint8_t ch0_ref_ch2 	: 1;
					uint8_t ch0_ref_ch3 	: 1;
					uint8_t ch0_ref_ch4 	: 1;
					uint8_t ch0_ref_ch5 	: 1;
					uint8_t ch0_ref_ch6 	: 1;
					uint8_t ch0_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch0;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch0_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch0;
			};
		};
		uint8_t buffer[14];
} IQS269_CH0_SETTINGS;
#pragma pack(4)

/* Channel 1 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch1_crx0_en 		: 1; //0
					uint8_t ch1_crx1_en 		: 1;
					uint8_t ch1_crx2_en 		: 1;
					uint8_t ch1_crx3_en 		: 1;
					uint8_t ch1_crx4_en 		: 1;
					uint8_t ch1_crx5_en 		: 1;
					uint8_t ch1_crx6_en 		: 1;
					uint8_t ch1_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch1;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch1_ctx0_en 		: 1; //0
					uint8_t ch1_ctx1_en 		: 1; //0
					uint8_t ch1_ctx2_en 		: 1; //0
					uint8_t ch1_ctx3_en 		: 1; //0
					uint8_t ch1_ctx4_en 		: 1; //0
					uint8_t ch1_ctx5_en 		: 1; //0
					uint8_t ch1_ctx6_en 		: 1; //0
					uint8_t ch1_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch1;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch1_ati_mode 	  : 2;
					uint8_t ch1_proxctrl_res1 : 2;
					uint8_t ch1_int_cap_size  : 1;
					uint8_t ch1_proxctrl_adv1 : 1;
					uint8_t ch1_proxctrl_adv2 : 1;
					uint8_t ch1_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch1;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch1_sensor_mode 	  : 4;
					uint8_t ch1_proxcfg_res1 	  : 1;
					uint8_t ch1_proj_bias_cur     : 2;
					uint8_t ch1_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch1;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch1_stat_fe			   : 1;
					uint8_t ch1_sense_freq		   : 2;
					uint8_t ch1_proxcfg1_res1	   : 1;
					uint8_t ch1_proxcfg1_res2	   : 1;
					uint8_t ch1_en_int_cap		   : 1;
					uint8_t ch1_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch1;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch1_ati_target	: 6;
					uint8_t ch1_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch1;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch1_fine_op		: 4;
					uint8_t ch1_coarse_op	: 2;
					uint8_t	ch1_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch1;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch1_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch1;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch1_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch1;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch1_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch1;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch1_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch1;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch1_touch_hyst 	: 4;
					uint8_t ch1_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch1;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch1_ref_ch0 	: 1;
					uint8_t ch1_ref_ch1 	: 1;
					uint8_t ch1_ref_ch2 	: 1;
					uint8_t ch1_ref_ch3 	: 1;
					uint8_t ch1_ref_ch4 	: 1;
					uint8_t ch1_ref_ch5 	: 1;
					uint8_t ch1_ref_ch6 	: 1;
					uint8_t ch1_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch1;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch1_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch1;
			};
		};
		uint8_t buffer[14];
} IQS269_CH1_SETTINGS;
#pragma pack(4)

/* Channel 2 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch2_crx0_en 		: 1; //0
					uint8_t ch2_crx1_en 		: 1;
					uint8_t ch2_crx2_en 		: 1;
					uint8_t ch2_crx3_en 		: 1;
					uint8_t ch2_crx4_en 		: 1;
					uint8_t ch2_crx5_en 		: 1;
					uint8_t ch2_crx6_en 		: 1;
					uint8_t ch2_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch2;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch2_ctx0_en 		: 1; //0
					uint8_t ch2_ctx1_en 		: 1; //0
					uint8_t ch2_ctx2_en 		: 1; //0
					uint8_t ch2_ctx3_en 		: 1; //0
					uint8_t ch2_ctx4_en 		: 1; //0
					uint8_t ch2_ctx5_en 		: 1; //0
					uint8_t ch2_ctx6_en 		: 1; //0
					uint8_t ch2_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch2;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch2_ati_mode 	  : 2;
					uint8_t ch2_proxctrl_res1 : 2;
					uint8_t ch2_int_cap_size  : 1;
					uint8_t ch2_proxctrl_adv1 : 1;
					uint8_t ch2_proxctrl_adv2 : 1;
					uint8_t ch2_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch2;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch2_sensor_mode 	  : 4;
					uint8_t ch2_proxcfg_res1 	  : 1;
					uint8_t ch2_proj_bias_cur     : 2;
					uint8_t ch2_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch2;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch2_stat_fe			   : 1;
					uint8_t ch2_sense_freq		   : 2;
					uint8_t ch2_proxcfg1_res1	   : 1;
					uint8_t ch2_proxcfg1_res2	   : 1;
					uint8_t ch2_en_int_cap		   : 1;
					uint8_t ch2_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch2;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch2_ati_target	: 6;
					uint8_t ch2_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch2;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch2_fine_op		: 4;
					uint8_t ch2_coarse_op	: 2;
					uint8_t	ch2_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch2;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch2_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch2;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch2_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch2;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch2_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch2;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch2_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch2;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch2_touch_hyst 	: 4;
					uint8_t ch2_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch2;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch2_ref_ch0 	: 1;
					uint8_t ch2_ref_ch1 	: 1;
					uint8_t ch2_ref_ch2 	: 1;
					uint8_t ch2_ref_ch3 	: 1;
					uint8_t ch2_ref_ch4 	: 1;
					uint8_t ch2_ref_ch5 	: 1;
					uint8_t ch2_ref_ch6 	: 1;
					uint8_t ch2_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch2;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch2_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch2;
			};
		};
		uint8_t buffer[14];
} IQS269_CH2_SETTINGS;
#pragma pack(4)

/* Channel 3 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch3_crx0_en 		: 1; //0
					uint8_t ch3_crx1_en 		: 1;
					uint8_t ch3_crx2_en 		: 1;
					uint8_t ch3_crx3_en 		: 1;
					uint8_t ch3_crx4_en 		: 1;
					uint8_t ch3_crx5_en 		: 1;
					uint8_t ch3_crx6_en 		: 1;
					uint8_t ch3_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch3;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch3_ctx0_en 		: 1; //0
					uint8_t ch3_ctx1_en 		: 1; //0
					uint8_t ch3_ctx2_en 		: 1; //0
					uint8_t ch3_ctx3_en 		: 1; //0
					uint8_t ch3_ctx4_en 		: 1; //0
					uint8_t ch3_ctx5_en 		: 1; //0
					uint8_t ch3_ctx6_en 		: 1; //0
					uint8_t ch3_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch3;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch3_ati_mode 	  : 2;
					uint8_t ch3_proxctrl_res1 : 2;
					uint8_t ch3_int_cap_size  : 1;
					uint8_t ch3_proxctrl_adv1 : 1;
					uint8_t ch3_proxctrl_adv2 : 1;
					uint8_t ch3_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch3;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch3_sensor_mode 	  : 4;
					uint8_t ch3_proxcfg_res1 	  : 1;
					uint8_t ch3_proj_bias_cur     : 2;
					uint8_t ch3_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch3;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch3_stat_fe			   : 1;
					uint8_t ch3_sense_freq		   : 2;
					uint8_t ch3_proxcfg1_res1	   : 1;
					uint8_t ch3_proxcfg1_res2	   : 1;
					uint8_t ch3_en_int_cap		   : 1;
					uint8_t ch3_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch3;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch3_ati_target	: 6;
					uint8_t ch3_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch3;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch3_fine_op		: 4;
					uint8_t ch3_coarse_op	: 2;
					uint8_t	ch3_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch3;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch3_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch3;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch3_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch3;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch3_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch3;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch3_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch3;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch3_touch_hyst 	: 4;
					uint8_t ch3_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch3;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch3_ref_ch0 	: 1;
					uint8_t ch3_ref_ch1 	: 1;
					uint8_t ch3_ref_ch2 	: 1;
					uint8_t ch3_ref_ch3 	: 1;
					uint8_t ch3_ref_ch4 	: 1;
					uint8_t ch3_ref_ch5 	: 1;
					uint8_t ch3_ref_ch6 	: 1;
					uint8_t ch3_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch3;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch3_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch3;
			};
		};
		uint8_t buffer[14];
} IQS269_CH3_SETTINGS;
#pragma pack(4)

/* Channel 4 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch4_crx0_en 		: 1; //0
					uint8_t ch4_crx1_en 		: 1;
					uint8_t ch4_crx2_en 		: 1;
					uint8_t ch4_crx3_en 		: 1;
					uint8_t ch4_crx4_en 		: 1;
					uint8_t ch4_crx5_en 		: 1;
					uint8_t ch4_crx6_en 		: 1;
					uint8_t ch4_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch4;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch4_ctx0_en 		: 1; //0
					uint8_t ch4_ctx1_en 		: 1; //0
					uint8_t ch4_ctx2_en 		: 1; //0
					uint8_t ch4_ctx3_en 		: 1; //0
					uint8_t ch4_ctx4_en 		: 1; //0
					uint8_t ch4_ctx5_en 		: 1; //0
					uint8_t ch4_ctx6_en 		: 1; //0
					uint8_t ch4_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch4;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch4_ati_mode 	  : 2;
					uint8_t ch4_proxctrl_res1 : 2;
					uint8_t ch4_int_cap_size  : 1;
					uint8_t ch4_proxctrl_adv1 : 1;
					uint8_t ch4_proxctrl_adv2 : 1;
					uint8_t ch4_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch4;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch4_sensor_mode 	  : 4;
					uint8_t ch4_proxcfg_res1 	  : 1;
					uint8_t ch4_proj_bias_cur     : 2;
					uint8_t ch4_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch4;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch4_stat_fe			   : 1;
					uint8_t ch4_sense_freq		   : 2;
					uint8_t ch4_proxcfg1_res1	   : 1;
					uint8_t ch4_proxcfg1_res2	   : 1;
					uint8_t ch4_en_int_cap		   : 1;
					uint8_t ch4_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch4;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch4_ati_target	: 6;
					uint8_t ch4_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch4;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch4_fine_op		: 4;
					uint8_t ch4_coarse_op	: 2;
					uint8_t	ch4_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch4;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch4_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch4;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch4_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch4;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch4_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch4;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch4_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch4;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch4_touch_hyst 	: 4;
					uint8_t ch4_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch4;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch4_ref_ch0 	: 1;
					uint8_t ch4_ref_ch1 	: 1;
					uint8_t ch4_ref_ch2 	: 1;
					uint8_t ch4_ref_ch3 	: 1;
					uint8_t ch4_ref_ch4 	: 1;
					uint8_t ch4_ref_ch5 	: 1;
					uint8_t ch4_ref_ch6 	: 1;
					uint8_t ch4_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch4;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch4_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch4;
			};
		};
		uint8_t buffer[14];
} IQS269_CH4_SETTINGS;
#pragma pack(4)

/* Channel 5 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch5_crx0_en 		: 1; //0
					uint8_t ch5_crx1_en 		: 1;
					uint8_t ch5_crx2_en 		: 1;
					uint8_t ch5_crx3_en 		: 1;
					uint8_t ch5_crx4_en 		: 1;
					uint8_t ch5_crx5_en 		: 1;
					uint8_t ch5_crx6_en 		: 1;
					uint8_t ch5_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch5;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch5_ctx0_en 		: 1; //0
					uint8_t ch5_ctx1_en 		: 1; //0
					uint8_t ch5_ctx2_en 		: 1; //0
					uint8_t ch5_ctx3_en 		: 1; //0
					uint8_t ch5_ctx4_en 		: 1; //0
					uint8_t ch5_ctx5_en 		: 1; //0
					uint8_t ch5_ctx6_en 		: 1; //0
					uint8_t ch5_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch5;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch5_ati_mode 	  : 2;
					uint8_t ch5_proxctrl_res1 : 2;
					uint8_t ch5_int_cap_size  : 1;
					uint8_t ch5_proxctrl_adv1 : 1;
					uint8_t ch5_proxctrl_adv2 : 1;
					uint8_t ch5_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch5;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch5_sensor_mode 	  : 4;
					uint8_t ch5_proxcfg_res1 	  : 1;
					uint8_t ch5_proj_bias_cur     : 2;
					uint8_t ch5_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch5;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch5_stat_fe			   : 1;
					uint8_t ch5_sense_freq		   : 2;
					uint8_t ch5_proxcfg1_res1	   : 1;
					uint8_t ch5_proxcfg1_res2	   : 1;
					uint8_t ch5_en_int_cap		   : 1;
					uint8_t ch5_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch5;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch5_ati_target	: 6;
					uint8_t ch5_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch5;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch5_fine_op		: 4;
					uint8_t ch5_coarse_op	: 2;
					uint8_t	ch5_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch5;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch5_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch5;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch5_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch5;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch5_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch5;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch5_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch5;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch5_touch_hyst 	: 4;
					uint8_t ch5_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch5;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch5_ref_ch0 	: 1;
					uint8_t ch5_ref_ch1 	: 1;
					uint8_t ch5_ref_ch2 	: 1;
					uint8_t ch5_ref_ch3 	: 1;
					uint8_t ch5_ref_ch4 	: 1;
					uint8_t ch5_ref_ch5 	: 1;
					uint8_t ch5_ref_ch6 	: 1;
					uint8_t ch5_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch5;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch5_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch5;
			};
		};
		uint8_t buffer[14];
} IQS269_CH5_SETTINGS;
#pragma pack(4)

/* Channel 6 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch6_crx0_en 		: 1; //0
					uint8_t ch6_crx1_en 		: 1;
					uint8_t ch6_crx2_en 		: 1;
					uint8_t ch6_crx3_en 		: 1;
					uint8_t ch6_crx4_en 		: 1;
					uint8_t ch6_crx5_en 		: 1;
					uint8_t ch6_crx6_en 		: 1;
					uint8_t ch6_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch6;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch6_ctx0_en 		: 1; //0
					uint8_t ch6_ctx1_en 		: 1; //0
					uint8_t ch6_ctx2_en 		: 1; //0
					uint8_t ch6_ctx3_en 		: 1; //0
					uint8_t ch6_ctx4_en 		: 1; //0
					uint8_t ch6_ctx5_en 		: 1; //0
					uint8_t ch6_ctx6_en 		: 1; //0
					uint8_t ch6_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch6;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch6_ati_mode 	  : 2;
					uint8_t ch6_proxctrl_res1 : 2;
					uint8_t ch6_int_cap_size  : 1;
					uint8_t ch6_proxctrl_adv1 : 1;
					uint8_t ch6_proxctrl_adv2 : 1;
					uint8_t ch6_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch6;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch6_sensor_mode 	  : 4;
					uint8_t ch6_proxcfg_res1 	  : 1;
					uint8_t ch6_proj_bias_cur     : 2;
					uint8_t ch6_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch6;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch6_stat_fe			   : 1;
					uint8_t ch6_sense_freq		   : 2;
					uint8_t ch6_proxcfg1_res1	   : 1;
					uint8_t ch6_proxcfg1_res2	   : 1;
					uint8_t ch6_en_int_cap		   : 1;
					uint8_t ch6_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch6;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch6_ati_target	: 6;
					uint8_t ch6_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch6;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch6_fine_op		: 4;
					uint8_t ch6_coarse_op	: 2;
					uint8_t	ch6_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch6;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch6_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch6;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch6_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch6;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch6_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch6;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch6_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch6;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch6_touch_hyst 	: 4;
					uint8_t ch6_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch6;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch6_ref_ch0 	: 1;
					uint8_t ch6_ref_ch1 	: 1;
					uint8_t ch6_ref_ch2 	: 1;
					uint8_t ch6_ref_ch3 	: 1;
					uint8_t ch6_ref_ch4 	: 1;
					uint8_t ch6_ref_ch5 	: 1;
					uint8_t ch6_ref_ch6 	: 1;
					uint8_t ch6_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch6;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch6_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch6;
			};
		};
		uint8_t buffer[14];
} IQS269_CH6_SETTINGS;
#pragma pack(4)

/* Channel 7 Settings  */
#pragma pack(1)
typedef union {
		struct {

			// Channel CRX (sensing pin) Enable
			union
			{
				struct
				{
					uint8_t ch7_crx0_en 		: 1; //0
					uint8_t ch7_crx1_en 		: 1;
					uint8_t ch7_crx2_en 		: 1;
					uint8_t ch7_crx3_en 		: 1;
					uint8_t ch7_crx4_en 		: 1;
					uint8_t ch7_crx5_en 		: 1;
					uint8_t ch7_crx6_en 		: 1;
					uint8_t ch7_crx7_en 		: 1;//7
				};
				uint8_t iqs269_pxs_crxsel_ch7;
			};

			// Channel TX (transmit pin) enable
			union
			{
				struct
				{
					uint8_t ch7_ctx0_en 		: 1; //0
					uint8_t ch7_ctx1_en 		: 1; //0
					uint8_t ch7_ctx2_en 		: 1; //0
					uint8_t ch7_ctx3_en 		: 1; //0
					uint8_t ch7_ctx4_en 		: 1; //0
					uint8_t ch7_ctx5_en 		: 1; //0
					uint8_t ch7_ctx6_en 		: 1; //0
					uint8_t ch7_ctx7_en 		: 1; //0

				};
				uint8_t iqs269_pxs_ctxsel_ch7;
			};

			// Channel Sensing engine settings 1
			union
			{
				struct
				{
					uint8_t ch7_ati_mode 	  : 2;
					uint8_t ch7_proxctrl_res1 : 2;
					uint8_t ch7_int_cap_size  : 1;
					uint8_t ch7_proxctrl_adv1 : 1;
					uint8_t ch7_proxctrl_adv2 : 1;
					uint8_t ch7_proxctrl_adv3 : 1;
				};
				uint8_t iqs269_pxs_proxctrl_ch7;
			};

			// Channel Sensing engine settings 2
			union
			{
				struct
				{
					uint8_t ch7_sensor_mode 	  : 4;
					uint8_t ch7_proxcfg_res1 	  : 1;
					uint8_t ch7_proj_bias_cur     : 2;
					uint8_t ch7_proxcfg_adv1 	  : 1;
				};
				uint8_t iqs269_pxs_proxcfg0_ch7;
			};

			// Channel Sensing engine settings 3
			union
			{
				struct
				{
					uint8_t ch7_stat_fe			   : 1;
					uint8_t ch7_sense_freq		   : 2;
					uint8_t ch7_proxcfg1_res1	   : 1;
					uint8_t ch7_proxcfg1_res2	   : 1;
					uint8_t ch7_en_int_cap		   : 1;
					uint8_t ch7_proxcfg1_res3	   : 2;

				};
				uint8_t iqs269_pxs_proxcfg1_testreg0_ch7;
			};


			// Auto Tuning Implementation (ATI) base value target and count target
			union
			{
				struct
				{
					uint8_t ch7_ati_target	: 6;
					uint8_t ch7_ati_base    : 2;
				};
				uint8_t iqs269_ati_base_and_target_ch7;
			};

			// Channel Multiplier Setting - normal use read only
			union
			{
				struct
				{
					uint8_t ch7_fine_op		: 4;
					uint8_t ch7_coarse_op	: 2;
					uint8_t	ch7_comp_msb	: 2;
				};
				uint8_t iqs269_ati_mirror_ch7;
			};

			// Compensation (ATI) - normal use read only
			union
			{
				struct
				{
					uint8_t ch7_comp_lsb 	: 8;
				};
				uint8_t iqs269_ati_pcc_ch7;
			};

			// Channel Proximity Threshold (4 sample debounce_ = 0 -255 counts
			union
			{
				struct
				{
					uint8_t ch7_prox_thresh 	: 8;
				};
				uint8_t iqs269_pxs_prox_threshold_ch7;
			};

			// Channel Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch7_touch_thresh 	: 8;
				};
				uint8_t iqs269_pxs_touch_threshold_ch7;
			};

			// Channel Deep Touch Threshold (x/256 of LTA value)
			union
			{
				struct
				{
					uint8_t ch7_deep_thresh 	: 8;
				};
				uint8_t iqs269_pxs_deep_threshold_ch7;
			};

			// Channel Hysteresis for Deep Touch and Touch
			union
			{
				struct
				{
					uint8_t ch7_touch_hyst 	: 4;
					uint8_t ch7_deep_hyst	: 4;
				};
				uint8_t iqs269_pxs_hysteresis_ch7;
			};

			// Reference channel association
			union
			{
				struct
				{
					uint8_t ch7_ref_ch0 	: 1;
					uint8_t ch7_ref_ch1 	: 1;
					uint8_t ch7_ref_ch2 	: 1;
					uint8_t ch7_ref_ch3 	: 1;
					uint8_t ch7_ref_ch4 	: 1;
					uint8_t ch7_ref_ch5 	: 1;
					uint8_t ch7_ref_ch6 	: 1;
					uint8_t ch7_ref_ch7 	: 1;
				};
				uint8_t iqs269_dcf_chb_association_ch7;
			};

			// Associated sensing channel impact weight (if this channel is associated to reference channel – 0 = no impact, 255 = 200% impact)
			union
			{
				struct
				{
					uint8_t ch7_dcf_weight 	: 8;
				};
				uint8_t iqs269_dcf_weight_ch7;
			};
		};
		uint8_t buffer[14];
} IQS269_CH7_SETTINGS;
#pragma pack(4)

/* I2C Control Settings */
#pragma pack(1)
typedef union{
			struct
			{
				union
				{
					struct
					{
						uint8_t i2c_reserved_1			: 4;
						uint8_t i2c_sleep_ready			: 1;
						uint8_t i2c_disable_read_only	: 1;
						uint8_t i2c_disable_stop		: 1;
						uint8_t i2c_end_window			: 1;
					};
					uint8_t iqs269_i2c_control_set;
				};

				union
				{
					struct
					{
						uint8_t i2c_reserved 		: 8;
					};
					uint8_t iqs269_i2c_reserved;
				};

			};
			uint8_t buffer[2];
} IQS269_I2C_CONTROL;
#pragma pack(4)


/* HALL UI Enable */
#pragma pack(1)
typedef union{
			struct
			{
				union
				{
					struct
					{
						uint8_t hall_reserved			: 7;
						uint8_t hall_ui_enable			: 1;
					};
					uint8_t iqs269_hall_ui_enable;
				};

				union
				{
					struct
					{
						uint8_t hall_reserved_1 		: 8;
					};
					uint8_t iqs269_hall_reserved_1;
				};

			};
			uint8_t buffer[2];
} IQS269_HALL_UI_ENABLE;
#pragma pack(4)


#pragma pack(1)
typedef union {
	struct
	{
		IQS269_PROD_INFO		  iqs269_prod_info;
		IQS269_SYS_GE_FLAGS		  iqs269_sys_ge_flags;
		IQS269_GESTURE_EV		  iqs269_gesture_ev;
		IQS269_CHAN_STATES		  iqs269_chan_states;
		IQS269_FILTERED_COUNTS	  iqs269_filtered_counts;
		IQS269_DELTA_COUNTS		  iqs269_dta;
		IQS269_REF_CHAN_DELTA	  iqs269_ref_chan_delta;
		IQS269_RESERVED			  iqs269_reserved;
		IQS269_SLIDER_CO		  iqs269_slider_co;
		IQS269_ABS_CAP			  iqs269_abs_cap;
		IQS269_HALL_CALL		  iqs269_hall_call;
		IQS269_POWER_SYS_SETTINGS iqs269_pow_sys_settings;
		IQS269_REPORT_AND_TIMINGS iqs269_rep_timing;
		IQS269_GLOBAL_SETTINGS	  iqs269_global_settings;
		IQS269_CH0_SETTINGS		  iqs269_ch0_settings;
		IQS269_CH1_SETTINGS		  iqs269_ch1_settings;
		IQS269_CH2_SETTINGS       iqs269_ch2_settings;
		IQS269_CH3_SETTINGS       iqs269_ch3_settings;
		IQS269_CH4_SETTINGS       iqs269_ch4_settings;
		IQS269_CH5_SETTINGS       iqs269_ch5_settings;
		IQS269_CH6_SETTINGS       iqs269_ch6_settings;
		IQS269_CH7_SETTINGS       iqs269_ch7_settings;
		IQS269_I2C_CONTROL		  iqs269_i2c_control;
		IQS269_HALL_UI_ENABLE	  iqs269_hall_ui_en;


	};
	uint8_t buffer[250];

} IQS269_MEMORY_MAP;
#pragma pack(4)


#pragma pack(1)
typedef struct {
	void		*section_pointer;
	uint8_t		starting_register;
	uint8_t		length;
	} IQS_memory_section;
#pragma pack(4)

#pragma pack(1)
typedef struct {
	IQS_memory_section iqs269_prod_info_memsec;
	IQS_memory_section iqs269_sys_ge_flags_memsec;
	IQS_memory_section iqs269_gesture_ev_memsec;
	IQS_memory_section iqs269_chan_states_memsec;
	IQS_memory_section iqs269_filtered_counts_memsec;
	IQS_memory_section iqs269_dta_memsec;
	IQS_memory_section iqs269_ref_chan_delta_memsec;
	IQS_memory_section iqs269_reserved_memsec;
	IQS_memory_section iqs269_slider_co_memsec;
	IQS_memory_section iqs269_abs_cap_memsec;
	IQS_memory_section iqs269_hall_call_memsec;
	IQS_memory_section iqs269_pow_sys_settings_memsec;
	IQS_memory_section iqs269_rep_timing_memsec;
	IQS_memory_section iqs269_global_settings_memsec;
	IQS_memory_section iqs269_ch0_settings_memsec;
	IQS_memory_section iqs269_ch1_settings_memsec;
	IQS_memory_section iqs269_ch2_settings_memsec;
	IQS_memory_section iqs269_ch3_settings_memsec;
	IQS_memory_section iqs269_ch4_settings_memsec;
	IQS_memory_section iqs269_ch5_settings_memsec;
	IQS_memory_section iqs269_ch6_settings_memsec;
	IQS_memory_section iqs269_ch7_settings_memsec;
	IQS_memory_section iqs269_i2c_control_memsec;
	IQS_memory_section iqs269_hall_ui_en_memsec;


} IQS269_MemSec;
#pragma pack(4)


/**
 * @brief  Type for IQS269
 */
#pragma pack(1)
typedef struct iqs269_s{
	IQS269_MemSec					iqs269_memsec;
	IQS269_MEMORY_MAP 				IQSMemoryMap;
	AZQ_Device_t					azq_iqs269;
	uint32_t						update_timestamp;								// the last time the IC was read
	bool							updated;										// flag to indicate that this is a new value
	IQS269_State_e					state;											// the state of the IQS621
	IQS269_init_e 					init_state;
	bool							retry_flag;
	bool							setting_upd;
	bool							openwindow;
	bool							jobdone;
	bool							reading;
	List_t							iqs269_queue;
	int8_t							queue_index;
}iqs269_s;
#pragma pack(4)



void init_iqs269(iqs269_s *iqs269);
bool run_init_iqs269(void);
void init_iqs269_MM(void);
void iqs269_set_retry_flag(void);
bool iqs269_service_update_flag(void);
bool iqs269_checkReadyPin(void);
void iqs269_open_window(void);
void iqs269_close_window(void);
void iqs269_start_job(void);
void iqs269_end_job(void);
void iqs269_trans_callback(void);
bool iqs269_rdy_callback(uint8_t IR);
IQS269_update_e run_iqs269(void);
bool iqs269_GestureEvent(void);
bool iqs269_TouchEvent(void);
bool iqs269_ProxEvent(void);
bool iqs269_PM_ChangeEvent(void);
bool iqs269_DeepTouchEvent(void);
uint16_t iqs269_readChannelCounts(IQS269_Channel_e channel);
uint16_t iqs269_readChannelLta(	IQS269_Channel_e channel);
int16_t iqs269_readChannelDelta(IQS269_Channel_e channel);
bool iqs269_atiChannels(uint8_t channelByte, IQS269_StopRestart_e stopOrRestart);
bool iqs269_Channel_TouchState(IQS269_Channel_e channel);
bool iqs269_Channel_DeepTouchState(IQS269_Channel_e channel);
bool iqs269_Channel_ProxState(IQS269_Channel_e channel);
bool iqs269_Channel_ProxDIState(IQS269_Channel_e channel);
IQS269_gesture_e iqs269_SliderEvent(IQS269_Slider_e slider);
uint8_t iqs269_SliderCoordinates(IQS269_Slider_e slider);
IQS269_PowerMode_e iqs269_GetPowerMode(void);
bool queue_iqs_transaction(I2C_TypeDef *I2Cx, uint8_t addr, uint8_t reg, uint8_t *buffer, uint8_t num_bytes, I2C_Read_Write_e type,
		void *start_callback, void *trans_callback, void *stop_callback, I2C_Last_e action, I2C_transaction_key_e key, I2C_trans_position_e position);
bool Run_IQS_Queue(I2C_transaction_s *temp_trans);
bool iqs269_QueueValueUpdates(void);
bool iqs269_deQueueValueUpdates(void);
bool iqs269_checkSoftReset(void);

void iqs269_abscap_MeasCap_Selection(IQS269_Cap_Meas_e Cx, IQS269_Fill_Cap_e C_fill, IQS269_StopRestart_e stopOrRestart);
uint32_t iqs269_getAbsCapMeasurement(IQS269_Cap_Meas_e Cx);

void iqs269_enable_AutoPower_switch(IQS269_StopRestart_e stopOrRestart);
void iqs269_disable_AutoPower_switch(IQS269_StopRestart_e stopOrRestart);
void iqs269_enableAbsCapUI(IQS269_StopRestart_e stopOrRestart);
void iqs269_disableAbsCapUI(IQS269_StopRestart_e stopOrRestart);
void iqs269_setAbsCapCH(IQS269_Channel_e channel, IQS269_StopRestart_e stopOrRestart);
bool iqs269_setFOSC(IQS269_SystemClock_e System_ClockSpeed, IQS269_StopRestart_e stopOrRestart);
void iqs269_setPowerMode(IQS269_PowerMode_e PowerMode, IQS269_StopRestart_e stopOrRestart);
void iqs269_setRefChannel(IQS269_Channel_e channel, uint8_t ref_channel_byte, IQS269_StopRestart_e stopOrRestart);
void iqs269_enableHallUI(IQS269_StopRestart_e stopOrRestart);
void iqs269_disableHallUI(IQS269_StopRestart_e stopOrRestart);
uint8_t iqs269_getProdNumber(void);
uint8_t iqs269_getSoftVersion(void);
uint8_t iqs269_getHardNumber(void);
void iqs269_QueueABSCapValueUpdates(void);
void iqs269_setbit_NPMode(void);
void iqs269_setbit_LPMode(void);
void iqs269_setbit_ULPMode(void);
void iqs269_setbit_HALTMode(void);
void iqs269_setbit_enableAbsCapCH0(void);
void iqs269_setbit_enableAbsCapCH1(void);
void iqs269_setbit_enableAbsCapCH2(void);
void iqs269_setbit_enableAbsCapCH3(void);
void iqs269_setbit_enableAbsCapCH4(void);
void iqs269_setbit_enableAbsCapCH5(void);
void iqs269_setbit_enableAbsCapCH6(void);
void iqs269_setbit_enableAbsCapCH7(void);
void iqs269_setbit_enableAbsCapUI(void);
void iqs269_setbit_disableAbsCapUI(void);
void iqs269_setbit_4MHzOSC(void);
void iqs269_setbit_16MHzOSC(void);
void iqs269_setbit_AutoPSwitch(void);
void iqs269_resetbit_AutoPSwitch(void);

void iqs269_setbit_enableHallUI(void);
void iqs269_resetbit_enableHallUI(void);
void iqs269_setbit_abscapRefCap1_1(void);
void iqs269_setbit_abscapRefCap1_2(void);
void iqs269_setbit_abscapRefCap1_3(void);
void iqs269_setbit_abscapRefCap1_4(void);

void iqs269_setbit_abscapRefCap2_1(void);
void iqs269_setbit_abscapRefCap2_2(void);
void iqs269_setbit_abscapRefCap2_3(void);
void iqs269_setbit_abscapRefCap2_4(void);

void iqs269_setbit_SoftReset(void);
void iqs269_resetbit_SoftReset(void);
void iqs269_setbit_ATI(void);
void iqs269_resetbit_ATI(void);

void iqs269_resetbit_ShowReset(void);
void iqs269_setbit_ShowReset(void);
void iqs269_resetbit_AckReset(void);
void iqs269_setbit_AckReset(void);

void iqs269_setReseedByte(uint8_t channelByte);
void iqs269_ReseedChannels(uint8_t channelByte, IQS269_StopRestart_e stopOrRestart);

/** @}  End Group IQS269 Functions
  */

/**	@}  End group azq_drivers_iqs269
	*/
  
#endif // __IQS269_H
