/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    draw.c						 				  	 	  	  	  *
 * @brief 	    LCD driver file							  		  	  	  	  *
 * @author 		GF Visagie - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    13/02/2020                                                    *
 ******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "draw.h"
#include "spi_dma.h"
#include "AZQ_util.h"

/*** Macros ***/
/* Private macro -------------------------------------------------------------*/
#define ABS(x)   ((x) > 0 ? (x) : -(x))

/* Private variables ---------------------------------------------------------*/
// Screen object
SSD1306_t *_ssdPtr;

uint8_t dummy_RX_Buffer[SSD1306_WIDTH * SSD1306_HEIGHT / 8] = {0};
uint8_t dummy_Byte = 0x00;
uint8_t SSD1306_Init_Buffer[28];
uint32_t displayTimer;

/**
 * @brief  		Initialize the pointer to the SSD1306_t instance
 *
 * @param  	SSD1306_t* : \b ssd1306 - Pointer to SSD1306_t instance
 *
 * @return 		None
 */
void ssd1306_Init(SSD1306_t *ssd1306)
{
	_ssdPtr = ssd1306;
}

/**
 @brief Reset LCD by setting and resetting the RES pin
 *
 * @param none
 *
 * @return none
 */
void ssd1306_Reset(void) {
	/* CS = High (not selected)	*/
	LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);

	/* Reset the OLED */
	LL_GPIO_ResetOutputPin(SPI_RES_PORT, SPI_RES_PIN);
	LL_mDelay(10);
	LL_GPIO_SetOutputPin(SPI_RES_PORT, SPI_RES_PIN);
	LL_mDelay(10);
}

/**
 @brief Send a byte to the command register
 *
 * @param uint8_t : \b byte -  Specifies the command byte
 *
 * @return true when command is written succesfully
 */
bool ssd1306_WriteCommand(uint8_t byte) {

	if (_ssdPtr->IF_State == LCD_IF_BUSY)
		return false;

	LL_GPIO_ResetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);	// select OLED
	LL_GPIO_ResetOutputPin(SPI_DC_PORT, SPI_DC_PIN);			// command
	spi_readwrite(1, &byte, 1, &dummy_Byte);

	_ssdPtr->IF_State = LCD_IF_BUSY;

	//  /* Wait for the SPI sequence to end */
	//while(is_spi_busy());

	//LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);		// un-select OLED

	return true;
}


/**
 @brief Send a buffer to the command register
 *
 * @param uint8_t* : \b buffer -  Specifies the command buffer
 * @param size_t : \b buff_size -  Specifies the size of the command buffer
 *
 * @return true when command buffer is written succesfully
 */
bool ssd1306_WriteCommandBuffer(uint8_t* buffer, size_t buff_size) {

	if (_ssdPtr->IF_State == LCD_IF_BUSY)
		return false;

	LL_GPIO_ResetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);	// select OLED
	LL_GPIO_ResetOutputPin(SPI_DC_PORT, SPI_DC_PIN);			// command
	spi_readwrite(buff_size, &buffer[0], buff_size, &dummy_RX_Buffer[0]);

	_ssdPtr->IF_State = LCD_IF_BUSY;

	//  /* Wait for the SPI sequence to end */
	//while(is_spi_busy());

	//LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);		// un-select OLED
	return true;
}


/**
 * @brief  		Write the data buffer using spi_readwrite
 *
 * @param  uint8_t* : \b buffer - Specifies the data buffer
 * @param  size_t : \b buff_size - Specifies the data buffer size
 *
 * @return 		true when data buffer is written succesfully
 */
bool ssd1306_WriteData(uint8_t* buffer, size_t buff_size) {

	if (_ssdPtr->IF_State == LCD_IF_BUSY)
		return false;
	LL_GPIO_ResetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);	// select OLED
	LL_GPIO_SetOutputPin(SPI_DC_PORT, SPI_DC_PIN);				// data
	spi_readwrite(buff_size, &buffer[0], buff_size, &dummy_RX_Buffer[0]);

	_ssdPtr->IF_State = LCD_IF_BUSY;

	//  /* Wait for the SPI sequence to end */
	//while(is_spi_busy());

	//LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);		// un-select OLED
	return true;
}



// Screenbuffer
static uint8_t SSD1306_Buffer[SSD1306_WIDTH * SSD1306_HEIGHT / 8];			// 128 * 64 = 8 192 / 8 = 1024

void ssd1306_Fill_Init_Buffer(void){
	SSD1306_Init_Buffer[0] = 0xAE; //display off

	SSD1306_Init_Buffer[1] = 0x20; //Set Memory Addressing Mode
	SSD1306_Init_Buffer[2] = 0x00; // 00b,Horizontal Addressing Mode; 01b,Vertical Addressing Mode;
	// 10b,Page Addressing Mode (RESET); 11b,Invalid

	SSD1306_Init_Buffer[3] = 0x20; //Set Page Start Address for Page Addressing Mode,0-7   0xB0


	#ifdef SSD1306_MIRROR_VERT
	SSD1306_Init_Buffer[4] = 0xC0; // Mirror vertically
	#else
	SSD1306_Init_Buffer[4] = 0xC8; //Set COM Output Scan Direction
	#endif

	SSD1306_Init_Buffer[5] = 0x00; //---set low column address
	SSD1306_Init_Buffer[6] = 0x10; //---set high column address

	SSD1306_Init_Buffer[7] = 0x40; //--set start line address - CHECK

	SSD1306_Init_Buffer[8] = 0x81; //--set contrast control register - CHECK
	SSD1306_Init_Buffer[9] = 0xFF;

#ifdef SSD1306_MIRROR_HORIZ
	SSD1306_Init_Buffer[10] = 0xA0; // Mirror horizontally
#else
	SSD1306_Init_Buffer[10] = 0xA1; //--set segment re-map 0 to 127 - CHECK
#endif

#ifdef SSD1306_INVERSE_COLOR
	SSD1306_Init_Buffer[11] = 0xA7; //--set inverse color
#else
	SSD1306_Init_Buffer[11] = 0xA6; //--set normal color
#endif

	SSD1306_Init_Buffer[12] = 0xA8; //--set multiplex ratio(1 to 64) - CHECK
	SSD1306_Init_Buffer[13] = 0x3F; //

	SSD1306_Init_Buffer[14] = 0xA4; //0xa4,Output follows RAM content;0xa5,Output ignores RAM content

	SSD1306_Init_Buffer[15] = 0xD3; //-set display offset - CHECK
	SSD1306_Init_Buffer[16] = 0x00; //-not offset

	SSD1306_Init_Buffer[17] = 0xD5; //--set display clock divide ratio/oscillator frequency
	SSD1306_Init_Buffer[18] = 0xF0; //--set divide ratio

	SSD1306_Init_Buffer[19] = 0xD9; //--set pre-charge period
	SSD1306_Init_Buffer[20] = 0x22; //

	SSD1306_Init_Buffer[21] = 0xDA; //--set com pins hardware configuration - CHECK
	SSD1306_Init_Buffer[22] = 0x12;

	SSD1306_Init_Buffer[23] = 0xDB; //--set vcomh
	SSD1306_Init_Buffer[24] = 0x20; //0x20,0.77xVcc

	SSD1306_Init_Buffer[25] = 0x8D; //--set DC-DC enable
	SSD1306_Init_Buffer[26] = 0x14; //
	SSD1306_Init_Buffer[27] = 0xAF; //--turn on SSD1306 panel
}

// Initialize the oled screen
void run_ssd1306_Init(void) {

	switch(_ssdPtr->init_state)
	{

	case LCD_INIT_NONE:

		_ssdPtr->init_state = LCD_INIT_RESET;

		//	_ssdPtr->State = LCD_UPDATE_NONE;

		//	timerLCD = HAL_GetTick();

		break;

	case LCD_INIT_RESET:

		// Reset OLED
		ssd1306_Reset();

		// Wait for the screen to boot
		LL_mDelay(100);

		_ssdPtr->init_state = LCD_INIT_WRITE_SETTINGS;

		break;

	case LCD_INIT_WRITE_SETTINGS:

		//Fill the init Buffer with the init commands
		ssd1306_Fill_Init_Buffer();

		// Init OLED
		if(ssd1306_WriteCommandBuffer(&SSD1306_Init_Buffer[0], sizeof(SSD1306_Init_Buffer)))
		{
			_ssdPtr->init_state = LCD_INIT_CLEAR_SCREEN;
		}

		break;

	case LCD_INIT_CLEAR_SCREEN:

		// Clear screen
		ssd1306_Fill(Black);

		_ssdPtr->update_state = LCD_UPDATE_IDLE;

		_ssdPtr->init_state = LCD_INIT_FINAL;

		break;

	case LCD_INIT_FINAL:

		// Set default values for screen object
		_ssdPtr->CurrentX = 0;
		_ssdPtr->CurrentY = 0;

		_ssdPtr->Initialized = 1;
		//		_ssdPtr->State = LCD_UPDATE_READY;

		_ssdPtr->init_state = LCD_INIT_DONE;

		break;

	case LCD_INIT_DONE:

		break;

	default:
		break;
	}
}


/*
 * @brief Fill the entire screen with the given color.
 * @param [SSD1306_COLOR] color : Color to fill the screen with. (Black - White)
 * @return None.
 *
 */
void ssd1306_Fill(SSD1306_COLOR color) {
	/* Set memory */
	uint32_t i;

	for(i = 0; i < sizeof(SSD1306_Buffer); i++) {
		SSD1306_Buffer[i] = (color == Black) ? 0x00 : 0xFF;
	}
}

/*
 * @brief Fill the screen with the given color within specified parameters.
 * @param [SSD1306_COLOR] x1 : x coordinate for start point of block (left border)
 * @param [SSD1306_COLOR] y1 : y coordinate for start point of block (upper border)
 * @param [SSD1306_COLOR] x2 : x coordinate end point of block (right border)
 * @param [SSD1306_COLOR] y2 : y coordinate end point of block (bottom border)
 * @param [SSD1306_COLOR] color : Color to fill the screen with. (Black - White)
 * @return None.
 *
 */
void ssd1306_Clear_region(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, SSD1306_COLOR color)
{
	uint16_t i;
	uint16_t j;
	// To rotate display by 180 degrees
	x1 = SSD1306_WIDTH - x1 - 1;
	y1 = SSD1306_HEIGHT - y1 - 1;
	x2 = SSD1306_WIDTH - x2;
	y2 = SSD1306_HEIGHT - y2;

	if(x1 >= SSD1306_WIDTH || y1 >= SSD1306_HEIGHT || x2 >= SSD1306_WIDTH || y2 >= SSD1306_HEIGHT)
	{
		// Don't write outside the buffer
		return;
	}
	//Check if pixel should be inverted
	if(_ssdPtr->Inverted)
	{
		color = (SSD1306_COLOR)!color;
	}

	switch(color)
	{
	case White:
		// Step through pixel by pixel and set colour for the region
		for(i = x2; i <= x1; i++)
		{
			for(j = y2; j <= y1; j++)
			{
				SSD1306_Buffer[i + (j / 8) * SSD1306_WIDTH] |= 1 << (j % 8);
			}
		}
		break;

	case Black:

		// Step through pixel by pixel and set colour for the region
		for(i = x2; i <= x1; i++)
		{
			for(j = y2; j <= y1; j++)
			{
				SSD1306_Buffer[i + (j / 8) * SSD1306_WIDTH] &= ~(1 << (j % 8));
			}
		}
		break;

	default:
		break;
	}
}

// Fill the buffer with the Azoteq splash screen
void ssd1306_LoadLogo(void) {
	/* Set memory */
	uint32_t i;
	for(i = 0; i < sizeof(SSD1306_Buffer); i++) {
		SSD1306_Buffer[i] = Azoteq_logo_iqs269[1023 - i];
	}
}

// Write the screenbuffer with changed to the screen
bool ssd1306_UpdateScreen(void) {

	uint8_t buff[3];

	buff[0] = 0x20;		//0xB0
	buff[1] = 0x00;
	buff[2] = 0x10;

	switch(_ssdPtr->update_state)
	{

	case LCD_UPDATE_NONE:

		_ssdPtr->update_state = LCD_UPDATE_IDLE;

		break;

	case LCD_UPDATE_IDLE:

		break;

	case LCD_UPDATE_START:

		if(ssd1306_WriteCommandBuffer(&buff[0], 3))
		{
			_ssdPtr->update_state = LCD_UPDATE_COMMAND;
		}

		break;

	case LCD_UPDATE_COMMAND:

		if(ssd1306_WriteData(&SSD1306_Buffer[0],sizeof(SSD1306_Buffer)))
		{
			_ssdPtr->update_state = LCD_UPDATE_DATA;
		}

		break;

	case LCD_UPDATE_DATA:

		if(_ssdPtr->IF_State != LCD_IF_BUSY)
		{
			_ssdPtr->update_state = LCD_UPDATE_DONE;
		}

		break;

	case LCD_UPDATE_DONE:

		_ssdPtr->update_state = LCD_UPDATE_IDLE;

		return true;

		break;


	}

	return false;


}

//    Draw one pixel in the screenbuffer
//    X => X Coordinate
//    Y => Y Coordinate
//    color => Pixel color
void ssd1306_DrawPixel(uint8_t x, uint8_t y, SSD1306_COLOR color) {

	// To rotate display by 180 degrees
	x = SSD1306_WIDTH - x - 1;
	y = SSD1306_HEIGHT - y - 1;

	if(x >= SSD1306_WIDTH || y >= SSD1306_HEIGHT) {
		// Don't write outside the buffer
		return;
	}
	//Check if pixel should be inverted
	if(_ssdPtr->Inverted)
	{
		color = (SSD1306_COLOR)!color;
	}

	switch(color)
	{
	case White:
		SSD1306_Buffer[x + (y / 8) * SSD1306_WIDTH] |= 1 << (y % 8);
		break;
	case Black:
		SSD1306_Buffer[x + (y / 8) * SSD1306_WIDTH] &= ~(1 << (y % 8));
		break;
	default:
		break;
	}


}

// Draw 1 char to the screen buffer
// ch       => char to be written
// Font     => Font used to write
// color    => Black or White
char ssd1306_WriteChar(char ch, FontDef Font, SSD1306_COLOR color) {
	uint32_t i, b, j;

	// Check if character is valid
	if (ch < 32 || ch > 126)
		return 0;

	// Check remaining space on current line
	if (SSD1306_WIDTH < (_ssdPtr->CurrentX + Font.FontWidth) ||
			SSD1306_HEIGHT < (_ssdPtr->CurrentY + Font.FontHeight))
	{
		// Not enough space on current line
		return 0;
	}

	// Use the font to write
	for(i = 0; i < Font.FontHeight; i++)
	{
		b = Font.data[(ch - 32) * Font.FontHeight + i];
		for(j = 0; j < Font.FontWidth; j++)
		{
			if((b << j) & 0x8000)
			{
				ssd1306_DrawPixel(_ssdPtr->CurrentX + j, (_ssdPtr->CurrentY + i), (SSD1306_COLOR) color);
			} else
			{
				ssd1306_DrawPixel(_ssdPtr->CurrentX + j, (_ssdPtr->CurrentY + i), (SSD1306_COLOR)!color);
			}
		}
	}

	// The current space is now taken
	_ssdPtr->CurrentX += Font.FontWidth;

	// Return written char for validation
	return ch;
}

// Write full string to screenbuffer
char ssd1306_WriteString(char* str, FontDef Font, SSD1306_COLOR color) {
	// Write until null-byte
	while (*str) {
		if (ssd1306_WriteChar(*str, Font, color) != *str) {
			// Char could not be written
			return *str;
		}

		// Next char
		str++;
	}

	// Everything ok
	return *str;
}

// Position the cursor
void ssd1306_SetCursor(uint8_t x, uint8_t y) {
	_ssdPtr->CurrentX = x;
	_ssdPtr->CurrentY = y;
}

void run_LCD(void){

	switch (_ssdPtr->State) {

	case LCD_NONE:

		_ssdPtr->State = LCD_INIT;

		break;

	case LCD_INIT:
		/* run init*/
		run_ssd1306_Init();

		if(_ssdPtr->init_state == LCD_INIT_DONE)
		{
			_ssdPtr->State = LCD_RUN;
		}

		break;

	case LCD_RUN:

		// Flush buffer to screen
		ssd1306_UpdateScreen();

		break;

	default:
		_ssdPtr->State = LCD_INIT;
		break;
	}

	/* check end of spi transmission */
	run_LCD_Interface();

}

void run_LCD_Interface(void){

	switch (_ssdPtr->IF_State) {


	case LCD_IF_NONE:

		break;

	case LCD_IF_IDLE:

		break;

	case LCD_IF_BUSY:
		if(!is_spi_busy()){
			LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);		// un-select OLED
			_ssdPtr->IF_State = LCD_IF_IDLE;
		}
		break;

	default:
		break;
	}
}

//Check if LCD init is done and that LCD is in IDLE state
// Update screen after displayTime since last update
bool ssd1306_CheckScreenReady(uint32_t displayTime) {

	if (_ssdPtr->init_state == LCD_INIT_DONE) {
		if((_ssdPtr->update_state == LCD_UPDATE_IDLE) && (HAL_CheckTick(displayTimer, displayTime, true)))
		{
			displayTimer = HAL_GetTick();
			_ssdPtr->update_state = LCD_UPDATE_START;
			return true;
		}
	}

	return false;

}

void SSD1306_DrawLine(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, SSD1306_COLOR c) {
	int16_t dx, dy, sx, sy, err, e2, i, tmp, counter;

//	DEBUG_PORT->ODR ^= DEBUG_PIN;
	/* Check for overflow */
	if (x0 >= SSD1306_WIDTH) {
		x0 = SSD1306_WIDTH - 1;
	}
	if (x1 >= SSD1306_WIDTH) {
		x1 = SSD1306_WIDTH - 1;
	}
	if (y0 >= SSD1306_HEIGHT) {
		y0 = SSD1306_HEIGHT - 1;
	}
	if (y1 >= SSD1306_HEIGHT) {
		y1 = SSD1306_HEIGHT - 1;
	}


	dx = (x0 < x1) ? (x1 - x0) : (x0 - x1);
	dy = (y0 < y1) ? (y1 - y0) : (y0 - y1);
	sx = (x0 < x1) ? 1 : -1;
	sy = (y0 < y1) ? 1 : -1;
	err = ((dx > dy) ? dx : -dy) / 2;

	if (dx == 0) {
		if (y1 < y0) {
			tmp = y1;
			y1 = y0;
			y0 = tmp;
		}

		if (x1 < x0) {
			tmp = x1;
			x1 = x0;
			x0 = tmp;
		}

		/* Vertical line */
		for (i = y0; i <= y1; i++) {
			ssd1306_DrawPixel(x0, i, c);
		}

		/* Return from function */
		return;
	}

	if (dy == 0) {
		if (y1 < y0) {
			tmp = y1;
			y1 = y0;
			y0 = tmp;
		}

		if (x1 < x0) {
			tmp = x1;
			x1 = x0;
			x0 = tmp;
		}

		/* Horizontal line */
		for (i = x0; i <= x1; i++) {
			ssd1306_DrawPixel(i, y0, c);
		}

		/* Return from function */
		return;
	}

	while (1)
	{
	counter ++;
		ssd1306_DrawPixel(x0, y0, c);
		if (x0 == x1 && y0 == y1) {
			break;
		}
		e2 = err;
		if (e2 > -dx) {
			err -= dy;
			x0 += sx;
		}
		if (e2 < dy) {
			err += dx;
			y0 += sy;
		}
	if(counter == 1000)
	{
		counter = 0;
		break;
	}
	}
}

void SSD1306_DrawRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1306_COLOR c) {
	/* Check input parameters */
	if (
			x >= SSD1306_WIDTH ||
			y >= SSD1306_HEIGHT
	) {
		/* Return error */
		return;
	}

	/* Check width and height */
	if ((x + w) >= SSD1306_WIDTH) {
		w = SSD1306_WIDTH - x;
	}
	if ((y + h) >= SSD1306_HEIGHT) {
		h = SSD1306_HEIGHT - y;
	}

	/* Draw 4 lines */
	SSD1306_DrawLine(x, y, x + w, y, c);         /* Top line */
	SSD1306_DrawLine(x, y + h, x + w, y + h, c); /* Bottom line */
	SSD1306_DrawLine(x, y, x, y + h, c);         /* Left line */
	SSD1306_DrawLine(x + w, y, x + w, y + h, c); /* Right line */
}

void SSD1306_DrawFilledRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, SSD1306_COLOR c) {
	uint8_t i;

	/* Check input parameters */
	if (
			x >= SSD1306_WIDTH ||
			y >= SSD1306_HEIGHT
	) {
		/* Return error */
		return;
	}

	/* Check width and height */
	if ((x + w) >= SSD1306_WIDTH) {
		w = SSD1306_WIDTH - x;
	}
	if ((y + h) >= SSD1306_HEIGHT) {
		h = SSD1306_HEIGHT - y;
	}

	/* Draw lines */
	for (i = 0; i <= h; i++) {
		/* Draw lines */
		SSD1306_DrawLine(x, y + i, x + w, y + i, c);
	}
}

void SSD1306_DrawTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1306_COLOR color) {
	/* Draw lines */
	SSD1306_DrawLine(x1, y1, x2, y2, color);
	SSD1306_DrawLine(x2, y2, x3, y3, color);
	SSD1306_DrawLine(x3, y3, x1, y1, color);
}


void SSD1306_DrawFilledTriangle(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, uint16_t x3, uint16_t y3, SSD1306_COLOR color) {
	int16_t deltax = 0, deltay = 0, x = 0, y = 0, xinc1 = 0, xinc2 = 0,
			yinc1 = 0, yinc2 = 0, den = 0, num = 0, numadd = 0, numpixels = 0,
			curpixel = 0;

	deltax = ABS(x2 - x1);
	deltay = ABS(y2 - y1);
	x = x1;
	y = y1;

	if (x2 >= x1) {
		xinc1 = 1;
		xinc2 = 1;
	} else {
		xinc1 = -1;
		xinc2 = -1;
	}

	if (y2 >= y1) {
		yinc1 = 1;
		yinc2 = 1;
	} else {
		yinc1 = -1;
		yinc2 = -1;
	}

	if (deltax >= deltay){
		xinc1 = 0;
		yinc2 = 0;
		den = deltax;
		num = deltax / 2;
		numadd = deltay;
		numpixels = deltax;
	} else {
		xinc2 = 0;
		yinc1 = 0;
		den = deltay;
		num = deltay / 2;
		numadd = deltax;
		numpixels = deltay;
	}

	for (curpixel = 0; curpixel <= numpixels; curpixel++) {
		SSD1306_DrawLine(x, y, x3, y3, color);

		num += numadd;
		if (num >= den) {
			num -= den;
			x += xinc1;
			y += yinc1;
		}
		x += xinc2;
		y += yinc2;
	}
}

void SSD1306_DrawCircle(int16_t x0, int16_t y0, int16_t r, SSD1306_COLOR c) {
	int16_t f = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x = 0;
	int16_t y = r;

	ssd1306_DrawPixel(x0, y0 + r, c);
	ssd1306_DrawPixel(x0, y0 - r, c);
	ssd1306_DrawPixel(x0 + r, y0, c);
	ssd1306_DrawPixel(x0 - r, y0, c);

	while (x < y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;

		ssd1306_DrawPixel(x0 + x, y0 + y, c);
		ssd1306_DrawPixel(x0 - x, y0 + y, c);
		ssd1306_DrawPixel(x0 + x, y0 - y, c);
		ssd1306_DrawPixel(x0 - x, y0 - y, c);

		ssd1306_DrawPixel(x0 + y, y0 + x, c);
		ssd1306_DrawPixel(x0 - y, y0 + x, c);
		ssd1306_DrawPixel(x0 + y, y0 - x, c);
		ssd1306_DrawPixel(x0 - y, y0 - x, c);
	}
}

void SSD1306_DrawFilledCircle(int16_t x0, int16_t y0, int16_t r, SSD1306_COLOR c) {
	int16_t f = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x = 0;
	int16_t y = r;

	ssd1306_DrawPixel(x0, y0 + r, c);
	ssd1306_DrawPixel(x0, y0 - r, c);
	ssd1306_DrawPixel(x0 + r, y0, c);
	ssd1306_DrawPixel(x0 - r, y0, c);
	SSD1306_DrawLine(x0 - r, y0, x0 + r, y0, c);

	while (x < y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;

		SSD1306_DrawLine(x0 - x, y0 + y, x0 + x, y0 + y, c);
		SSD1306_DrawLine(x0 + x, y0 - y, x0 - x, y0 - y, c);

		SSD1306_DrawLine(x0 + y, y0 + x, x0 - y, y0 + x, c);
		SSD1306_DrawLine(x0 + y, y0 - x, x0 - y, y0 - x, c);
	}
}

void drawRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, SSD1306_COLOR color) {
	// smarter version
	SSD1306_DrawLine(x+r  , y    , x+r+(w-2*r)-1, y, color); // Top
	SSD1306_DrawLine(x+r  , y+h-1, x+r+(w-2*r)-1, y+h-1, color); // Bottom
	SSD1306_DrawLine(x    , y+r  , x, y+r+(h-2*r)-1, color); // Left
	SSD1306_DrawLine(x+w-1, y+r  , x+w-1, y+r+(h-2*r)-1, color); // Right
	// draw four corners
	drawCircleHelper(x+r    , y+r    , r, 1, color);
	drawCircleHelper(x+w-r-1, y+r    , r, 2, color);
	drawCircleHelper(x+w-r-1, y+h-r-1, r, 4, color);
	drawCircleHelper(x+r    , y+h-r-1, r, 8, color);
}

// Fill a rounded rectangle
void fillRoundRect(int16_t x, int16_t y, int16_t w, int16_t h, int16_t r, SSD1306_COLOR color) {
	// smarter version
	SSD1306_DrawFilledRectangle(x+r, y, w-2*r, h-1, color);

	// draw four corners
	fillCircleHelper(x+w-r-1, y+r, r, 1, h-2*r-1, color);
	fillCircleHelper(x+r    , y+r, r, 2, h-2*r-1, color);
}

void drawCircleHelper( int16_t x0, int16_t y0, int16_t r, uint8_t cornername, SSD1306_COLOR color) {
	int16_t f     = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x     = 0;
	int16_t y     = r;

	while (x<y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f     += ddF_y;
		}
		x++;
		ddF_x += 2;
		f     += ddF_x;
		if (cornername & 0x4) {
			ssd1306_DrawPixel(x0 + x, y0 + y, color);
			ssd1306_DrawPixel(x0 + y, y0 + x, color);
		}
		if (cornername & 0x2) {
			ssd1306_DrawPixel(x0 + x, y0 - y, color);
			ssd1306_DrawPixel(x0 + y, y0 - x, color);
		}
		if (cornername & 0x8) {
			ssd1306_DrawPixel(x0 - y, y0 + x, color);
			ssd1306_DrawPixel(x0 - x, y0 + y, color);
		}
		if (cornername & 0x1) {
			ssd1306_DrawPixel(x0 - y, y0 - x, color);
			ssd1306_DrawPixel(x0 - x, y0 - y, color);
		}
	}
}

// Used to do roundrects
void fillCircleHelper(int16_t x0, int16_t y0, int16_t r, uint8_t cornername, int16_t delta, SSD1306_COLOR color) {

	int16_t f     = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x     = 0;
	int16_t y     = r;

	while (x<y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f     += ddF_y;
		}
		x++;
		ddF_x += 2;
		f     += ddF_x;

		if (cornername & 0x1) {
			//writeFastVLine(x0+x, y0-y, 2*y+1+delta, color);
			//writeFastVLine(x0+y, y0-x, 2*x+1+delta, color);
			SSD1306_DrawLine(x0+x, y0-y, x0+x, y0-y+(2*y+1+delta)-1, color);
			SSD1306_DrawLine(x0+y, y0-x, x0+y, y0-x+(2*x+1+delta)-1, color);
		}
		if (cornername & 0x2) {
			//writeFastVLine(x0-x, y0-y, 2*y+1+delta, color);
			//writeFastVLine(x0-y, y0-x, 2*x+1+delta, color);
			SSD1306_DrawLine(x0-x, y0-y, x0-x, y0-y+(2*y+1+delta)-1, color);
			SSD1306_DrawLine(x0-y, y0-x, x0-y, y0-x+(2*x+1+delta)-1, color);
		}
	}
}
