/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    spi_dma.c						 				  	 	  	  *
 * @brief 	    spi dma driver file							  		  	  	  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    06/02/2019                                                    *
 ******************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "spi_dma.h"
#include <stdbool.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* SPI state */
SPI_State_e	state = SPI_NONE;
uint32_t	last_spi_timestamp = 0;

/* transmission */
volatile uint8_t ubTransmissionComplete = 0;

/* reception */
volatile uint8_t ubReceptionComplete = 0;
/* Private function prototypes -----------------------------------------------*/
void     Configure_DMA(void);
void	 Configure_SPI1(void);

/* Private functions ---------------------------------------------------------*/

/**
 * @brief  spi readwrite function
 * @param  None
 * @retval None
 */
void init_spi(void)
{

	/* Configure the SPI1 parameters */
	Configure_SPI1();

	/* Configure DMA channels for the SPI1  */
	Configure_DMA();

}

/**
 * @brief  check if the spi sequence is busy
 * @param  None
 * @retval None
 */
bool is_spi_busy(void)
{
	/* check for SPI timeout */
	if (HAL_CheckTick(last_spi_timestamp, SPI_TIMEOUT, true))
	{
		state = SPI_IDLE;
	}


	if ((state == SPI_IDLE) || (state == SPI_DONE))
	{
		return false;

	}
	return true;


}

/**
 * @brief  This function configures SPI1.
 * @note  This function is used to :
 *        -1- Enables GPIO clock and configures the SPI1 pins.
 *        -2- Configure NVIC for SPI1.
 *        -3- Configure SPI1 functional parameters.
 * @note   Peripheral configuration is minimal configuration from reset values.
 *         Thus, some useless LL unitary functions calls below are provided as
 *         commented examples - setting is default configuration from reset.
 * @param  None
 * @retval None
 */
void Configure_SPI1(void)
{
	/* (1) Enables GPIO clock and configures the SPI1 pins ********************/
	/* Enable the peripheral clock of GPIOA */
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA);
	/* Enable the peripheral clock of GPIOB */
	LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOB);

	/* Configure CS NSS Pin */
	LL_GPIO_SetPinMode(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN, LL_GPIO_MODE_OUTPUT);
	LL_GPIO_SetPinOutputType(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN, LL_GPIO_OUTPUT_PUSHPULL);
	LL_GPIO_SetPinSpeed(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN, LL_GPIO_PULL_DOWN);
	// Set the SPI CS
	LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);

	/* Configure SCK Pin */
	LL_GPIO_SetPinMode(SPI_SCK_PORT, SPI_SCK_PIN, LL_GPIO_MODE_ALTERNATE);
	LL_GPIO_SetAFPin_0_7(SPI_SCK_PORT, SPI_SCK_PIN, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(SPI_SCK_PORT, SPI_SCK_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_SCK_PORT, SPI_SCK_PIN, LL_GPIO_PULL_DOWN);

	/* Configure MISO Pin */
	LL_GPIO_SetPinMode(SPI_MISO_PORT, SPI_MISO_PIN, LL_GPIO_MODE_ALTERNATE);
	LL_GPIO_SetAFPin_0_7(SPI_MISO_PORT, SPI_MISO_PIN, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(SPI_MISO_PORT, SPI_MISO_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_MISO_PORT, SPI_MISO_PIN, LL_GPIO_PULL_DOWN);

	/* Configure MOSI Pin */
	LL_GPIO_SetPinMode(SPI_MOSI_PORT, SPI_MOSI_PIN, LL_GPIO_MODE_ALTERNATE);
	LL_GPIO_SetAFPin_0_7(SPI_MOSI_PORT, SPI_MOSI_PIN, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(SPI_MOSI_PORT, SPI_MOSI_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_MOSI_PORT, SPI_MOSI_PIN, LL_GPIO_PULL_DOWN);

	/* Configure RES Pin */
	LL_GPIO_SetPinMode(SPI_RES_PORT, SPI_RES_PIN, LL_GPIO_MODE_OUTPUT);
	//LL_GPIO_SetAFPin_0_7(SPI_RES_PORT, SPI_RES_PIN, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(SPI_RES_PORT, SPI_RES_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_RES_PORT, SPI_RES_PIN, LL_GPIO_PULL_DOWN);

	/* Configure DC Pin */
	LL_GPIO_SetPinMode(SPI_DC_PORT, SPI_DC_PIN, LL_GPIO_MODE_OUTPUT);
	//LL_GPIO_SetAFPin_0_7(SPI_DC_PORT, SPI_DC_PIN, LL_GPIO_AF_0);
	LL_GPIO_SetPinSpeed(SPI_DC_PORT, SPI_DC_PIN, LL_GPIO_SPEED_FREQ_VERY_HIGH);
	LL_GPIO_SetPinPull(SPI_DC_PORT, SPI_DC_PIN, LL_GPIO_PULL_DOWN);

	/* (3) Configure SPI1 functional parameters ********************************/
	/* Enable the peripheral clock of GPIOB */
	LL_APB2_GRP1_EnableClock(LL_APB2_GRP1_PERIPH_SPI1);

	/* Configure SPI1 communication */
	LL_SPI_SetBaudRatePrescaler(SPI1, LL_SPI_BAUDRATEPRESCALER_DIV16);
	LL_SPI_SetTransferDirection(SPI1,LL_SPI_FULL_DUPLEX);
	LL_SPI_SetClockPhase(SPI1, LL_SPI_PHASE_1EDGE);
	LL_SPI_SetClockPolarity(SPI1, LL_SPI_POLARITY_LOW);
	/* Reset value is LL_SPI_MSB_FIRST */
	LL_SPI_SetTransferBitOrder(SPI1, LL_SPI_MSB_FIRST);
	LL_SPI_SetDataWidth(SPI1, LL_SPI_DATAWIDTH_8BIT);
	LL_SPI_SetNSSMode(SPI1, LL_SPI_NSS_SOFT);
	LL_SPI_SetRxFIFOThreshold(SPI1, LL_SPI_RX_FIFO_TH_QUARTER);
	LL_SPI_SetMode(SPI1, LL_SPI_MODE_MASTER);

	/* Enable SPI1 */
	LL_SPI_Enable(SPI1);

}

/**
 * @brief  This function configures the DMA1 Channel 1 to copy data from
 *         Flash memory(aSRC_Const_Buffer) to Internal SRAM(aDST_Buffer).
 * @note   This function is used to :
 *         -1- Enable DMA1 clock
 *         -2- Configure the DMA functionnal parameters
 *         -3- Configure NVIC for DMA transfer complete/error interrupts
 * @note   Peripheral configuration is minimal configuration from reset values.
 *         Thus, some useless LL unitary functions calls below are provided as
 *         commented examples - setting is default configuration from reset.
 * @param   None
 * @retval  None
 */
void Configure_DMA(void)
{
	/* (1) Enable the clock of DMA1 */
	LL_AHB1_GRP1_EnableClock(LL_AHB1_GRP1_PERIPH_DMA1);

	/* (2) Configure the DMA functionnal parameters */
	/* Configuration of the DMA parameters can be done using unitary functions or using the specific configure function */
	/* Unitary Functions */

	/* Configure Function */
	LL_DMA_ConfigTransfer(DMA1, LL_DMA_CHANNEL_1, LL_DMA_DIRECTION_PERIPH_TO_MEMORY |
			LL_DMA_PRIORITY_HIGH              |
			LL_DMA_MODE_NORMAL                |
			LL_DMA_PERIPH_NOINCREMENT         |
			LL_DMA_MEMORY_INCREMENT           |
			LL_DMA_PDATAALIGN_BYTE            |
			LL_DMA_MDATAALIGN_BYTE);
	/* Configure Function */
	LL_DMA_ConfigTransfer(DMA1, LL_DMA_CHANNEL_2, LL_DMA_DIRECTION_MEMORY_TO_PERIPH |
			LL_DMA_PRIORITY_HIGH              |
			LL_DMA_MODE_NORMAL                |
			LL_DMA_PERIPH_NOINCREMENT         |
			LL_DMA_MEMORY_INCREMENT           |
			LL_DMA_PDATAALIGN_BYTE            |
			LL_DMA_MDATAALIGN_BYTE);

	/* Configure the peripheral request */
	/* Reset Value is LL_DMA_REQUEST_0  */
	/* Equal to "LL_DMA_SetPeriphRequest(...)" but there is a bug in the LL library */
	//    DMA1_CSELR->CSELR |= 1 << DMA_CSELR_C5S_Pos;
	LL_DMA_SetPeriphRequest(DMA1, LL_DMA_CHANNEL_1, LL_DMAMUX_REQ_SPI1_RX);


	/* Equal to "LL_DMA_SetPeriphRequest(...)" but there is a bug in the LL library */
	//    DMA1_CSELR->CSELR |= 1 << DMA_CSELR_C5S_Pos;
	LL_DMA_SetPeriphRequest(DMA1, LL_DMA_CHANNEL_2, LL_DMAMUX_REQ_SPI1_TX);

	/* (3) Configure NVIC for DMA transfer complete/error interrupts */
	LL_DMA_EnableIT_TC(DMA1, LL_DMA_CHANNEL_1);
	LL_DMA_EnableIT_TE(DMA1, LL_DMA_CHANNEL_1);
	LL_DMA_EnableIT_TC(DMA1, LL_DMA_CHANNEL_2);
	LL_DMA_EnableIT_TE(DMA1, LL_DMA_CHANNEL_2);
	NVIC_SetPriority(DMA1_Channel1_IRQn, 2);
	NVIC_EnableIRQ(DMA1_Channel1_IRQn);
	NVIC_SetPriority(DMA1_Channel2_3_IRQn, 2);
	NVIC_EnableIRQ(DMA1_Channel2_3_IRQn);
	/* Disable DMA1 Tx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_2);

	/* Disable DMA1 Rx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_1);

	/* Disable DMA RX Interrupt */
	LL_SPI_DisableDMAReq_RX(SPI1);

	/* Disable DMA TX Interrupt */
	LL_SPI_DisableDMAReq_TX(SPI1);

	/* set the SPI state to IDLE */
	state = SPI_IDLE;

}


/**
 * @brief  spi readwrite function
 * @param  None
 * @retval None
 */
void spi_readwrite(uint16_t TX_DataLength, uint8_t *TX_DataBuffer, uint16_t RX_DataLength, uint8_t *RX_DataBuffer)
{
	/* Reset DMA Transfer flags */
	ubReceptionComplete = 0;
	ubTransmissionComplete = 0;

	uint16_t datalength = 0;

	/* Determine the Datalength */
	if (TX_DataLength > RX_DataLength)
	{
		datalength = TX_DataLength;
	}
	else
	{
		datalength = RX_DataLength;
	}

	/* Clear the SPI RXNE Flag - Errata Bug
	 * https://electronics.stackexchange.com/questions/127488/full-duplex-spi-master-using-dma-stm32f105
	 * RXNE flag is cleared by reading DR.
	 */
	uint32_t tmpreg = SPI1->DR;

	/* Prepare DMA */
	LL_DMA_ConfigAddresses(DMA1, LL_DMA_CHANNEL_1,
			LL_SPI_DMA_GetRegAddr(SPI1),
			(uint32_t)RX_DataBuffer,
			LL_DMA_GetDataTransferDirection(DMA1, LL_DMA_CHANNEL_1));
	LL_DMA_SetDataLength(DMA1, LL_DMA_CHANNEL_1, datalength);

	/* Equal to "LL_DMA_SetPeriphRequest(...)" but there is a bug in the LL library */
	//    DMA1_CSELR->CSELR |= 1 << DMA_CSELR_C5S_Pos;
	LL_DMA_SetPeriphRequest(DMA1, LL_DMA_CHANNEL_1, LL_DMAMUX_REQ_SPI1_RX);

	LL_DMA_ConfigAddresses(DMA1, LL_DMA_CHANNEL_2,
			(uint32_t)TX_DataBuffer,
			LL_SPI_DMA_GetRegAddr(SPI1),
			LL_DMA_GetDataTransferDirection(DMA1, LL_DMA_CHANNEL_2));
	LL_DMA_SetDataLength(DMA1, LL_DMA_CHANNEL_2, datalength);

	/* Equal to "LL_DMA_SetPeriphRequest(...)" but there is a bug in the LL library */
	//    DMA1_CSELR->CSELR |= 1 << DMA_CSELR_C5S_Pos;
	LL_DMA_SetPeriphRequest(DMA1, LL_DMA_CHANNEL_2, LL_DMAMUX_REQ_SPI1_TX);

	/* Lower the CS pin */
	reset_SPI_CS_NSS_pin();

	/* set the SPI state to WAIT FOR TX DMA to complete */
	state = SPI_WAIT_FOR_TX_DMA;

	/* update the spi timestamp */
	last_spi_timestamp = HAL_GetTick();

	/* Enable DMA Channels */
	LL_DMA_EnableChannel(DMA1, LL_DMA_CHANNEL_2);
	LL_DMA_EnableChannel(DMA1, LL_DMA_CHANNEL_1);

	/* Configure SPI1 DMA transfer interrupts */
	/* Enable DMA RX Interrupt */
	LL_SPI_EnableDMAReq_RX(SPI1);

	/* Enable DMA TX Interrupt */
	LL_SPI_EnableDMAReq_TX(SPI1);

}

/**
 * @brief  Function called from DMA1 IRQ Handler when Rx transfer is completed
 * @param  None
 * @retval None
 */
void DMA1_ReceiveComplete_Callback(void)
{
	/* DMA Rx transfer completed */
	ubReceptionComplete = 1;

	/* Disable DMA1 Tx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_2);

	/* Disable DMA1 Rx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_1);

	/* Disable DMA RX Interrupt */
	LL_SPI_DisableDMAReq_RX(SPI1);

	/* Disable DMA TX Interrupt */
	LL_SPI_DisableDMAReq_TX(SPI1);

	LL_DMA_ClearFlag_GI1(DMA1);
	LL_DMA_ClearFlag_TC1(DMA1);
	LL_DMA_ClearFlag_HT1(DMA1);

	LL_DMA_ClearFlag_GI2(DMA1);
	LL_DMA_ClearFlag_TC2(DMA1);
	LL_DMA_ClearFlag_HT2(DMA1);

	/* Raise the CS pin */
	set_SPI_CS_NSS_pin();

	/* set the SPI state to DONE */
	state = SPI_DONE;

	//#ifdef DEBUG_IO_LED_EN
	//	  LL_GPIO_ResetOutputPin(OUTPUT_DEBUG_PORT, OUTPUT_DEBUG_PIN);
	//#endif

}

/**
 * @brief  Function called from DMA1 IRQ Handler when Tx transfer is completed
 * @param  None
 * @retval None
 */
void DMA1_TransmitComplete_Callback(void)
{
	/* DMA Tx transfer completed */
	ubTransmissionComplete = 1;

	/* set the SPI state to WAIT_FOR_RX_DMA */
	state = SPI_WAIT_FOR_RX_DMA;
}

/**
 * @brief  Function called in case of error detected in SPI IT Handler
 * @param  None
 * @retval None
 */
void SPI1_TransferError_Callback(void)
{
	/* Disable DMA1 Rx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_1);

	/* Disable DMA1 Tx Channel */
	LL_DMA_DisableChannel(DMA1, LL_DMA_CHANNEL_2);

}

/**
 * @brief  set the CS NSS pin
 * @param  None
 * @retval None
 */
void set_SPI_CS_NSS_pin (void)
{
	// Set the SPI CS
	LL_GPIO_SetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);
}

/**
 * @brief  reset the CS NSS pin
 * @param  None
 * @retval None
 */
void reset_SPI_CS_NSS_pin (void)
{
	// Release the SPI CS
	LL_GPIO_ResetOutputPin(SPI_NSS_CS_PORT, SPI_NSS_CS_PIN);
}

/**
  * @brief  This function handles DMA1 interrupt request.
  * @param  None
  * @retval None
  */
void DMA_Channel1_IRQHandler(void)
{
	  /* channel 1 callbacks */
	  if(LL_DMA_IsActiveFlag_GI1(DMA1))
	  {
	    LL_DMA_ClearFlag_GI1(DMA1);
	    /* Call function Reception complete Callback */
	    DMA1_ReceiveComplete_Callback();
	  }

	  if(LL_DMA_IsActiveFlag_TC1(DMA1))
	  {
		LL_DMA_ClearFlag_TC1(DMA1);
	    LL_DMA_ClearFlag_GI1(DMA1);
	    /* Call function Reception complete Callback */
	    DMA1_ReceiveComplete_Callback();
	  }
	  else if(LL_DMA_IsActiveFlag_TE1(DMA1))
	  {
	    /* Call Error function */
	    SPI1_TransferError_Callback();
	  }
}

/**
  * @brief  This function handles DMA1 interrupt request.
  * @param  None
  * @retval None
  */
void DMA_Channel2_3_IRQHandler(void)
{

  /* channel 1 callbacks */
  if(LL_DMA_IsActiveFlag_GI2(DMA1))
  {
	LL_DMA_ClearFlag_GI2(DMA1);
	LL_DMA_ClearFlag_TC2(DMA1);
    /* Call function Transmission complete Callback */
    DMA1_TransmitComplete_Callback();
  }

  if(LL_DMA_IsActiveFlag_TC2(DMA1))
  {
	LL_DMA_ClearFlag_GI2(DMA1);
	LL_DMA_ClearFlag_TC2(DMA1);
    /* Call function Transmission complete Callback */
    DMA1_TransmitComplete_Callback();
  }
  else if(LL_DMA_IsActiveFlag_TE2(DMA1))
  {
    /* Call Error function */
    SPI1_TransferError_Callback();
  }
}