/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    List.c							 				  	 	      *
 * @brief 	    Generic C list									  		  	  *
 * @author 		AJ van der Merwe - Azoteq PTY Ltd                             *
 * @version 	V1.0.0                                                    	  *
 * @date 	    27/03/2017                                                    *
 ******************************************************************************/



/* Includes ------------------------------------------------------------------*/
#include <list.h>
#include "main.h"

/** Struct for generic node */

struct Node_s{
	uint8_t key;             // Implement a key for brevity
	void *data;              // Generic data
	struct Node_s *pNext;    // Pointer to next node
};


/**
 * @brief   Initialise the list. We need to know the size of data type of list
 * @param   *_this - pointer to the List_t
 * @param   memorySize - the size required for the List
 * @param   maxSize - the maximum size of the list - in order to shift out
 */
void List_init(List_t *_this, uint8_t memorySize, uint8_t maxSize)
{
	_this->listSize = 0;
	_this->memorySize = memorySize; // We need to store the memory size for adding stuff
	_this->maxListSize = maxSize;
	_this->pHead = _this->pTail = NULL;
}

/**
 * @brief   Add data to the front of list. This requires malloc.
 * @param   *_this - pointer to the List_t
 * @param   *data - pointer to the data that must be enqueued
 * @retval  true - success; false - failed
 */
bool List_insertStart(List_t *_this, uint8_t key, const void *data)
{
	/* Malloc ahead, disable the interrupts */
//	I2C_disable_irq();
	__disable_irq();


	Node_t *newNode;

	newNode = (Node_t *)malloc(sizeof(Node_t));

	// Did malloc fail?
	if(newNode == NULL)
	{
		/* Malloc past, re-enable the interrupts */
		__enable_irq();
//		I2C_enable_irq();
		return false;

	}

	newNode->data = malloc(_this->memorySize);  // Create memory for the data - HBD


	// Did data malloc fail? If malloc failed, we need to free the memory we already allocated
	if((newNode->data == NULL) || (_this->memorySize == 0))
	{
		/* Malloc past, re-enable the interrupts */
		__enable_irq();
//		I2C_enable_irq();

		free(newNode);
		return false;
	}

	// This is added in front so next is head
	newNode->pNext = _this->pHead;

	// Now copy data to the node data structure - memory is allocated already
	memcpy(newNode->data, data, _this->memorySize);

	// Add key to the new node
	newNode->key = key;

	//  Check whether this is the first node
	if(_this->listSize == 0)
	{
		// Point head and tail node to first node
		_this->pHead = _this->pTail = newNode;
	}
	// We reached the max so squeeze out the back
	else if(_this->listSize == _this->maxListSize)
	{
		// Delete last node before adding new node
		List_deleteLast(_this);
		// Otherwise we move the new node in front of head node
		newNode->pNext = _this->pHead;
		// Now point head at new node
		_this->pHead = newNode;
	}
	else {
		// Otherwise we move the new node in front of head node
		newNode->pNext = _this->pHead;
		// Now point head at new node
		_this->pHead = newNode;
	}

	// We increase list size
	_this->listSize++;

	/* Malloc past, re-enable the interrupts */
	__enable_irq();
//	I2C_enable_irq();

	// Success
	return true;
}

/**
 * @brief   Add data to the back of list. This requires malloc.
 * @param   *_this- pointer to the List_t
 * @param   *data - pointer to the data that must be enqueued
 * @retval  true - success; false - failed
 */
bool List_insertEnd(List_t *_this, uint8_t key, const void *data)
{
	Node_t *newNode;

	/* Disable interrupts */
	__disable_irq();

	newNode = (Node_t *)malloc(sizeof(Node_t));

	// Did malloc fail?
	if(newNode == NULL) return false;

	newNode->data = malloc(_this->memorySize);  // Create memory for the data - HBD

	// Did data malloc fail? If malloc failed, we need to free the memory we already allocated
	if((newNode->data == NULL) || (_this->memorySize == 0))
	{
		free(newNode);
		return false;
	}

	// This is added to the back so next is NULL
	newNode->pNext = NULL;

	// Now copy data to the node data structure - memory is allocated already
	memcpy(newNode->data, data, _this->memorySize);

	// Add key to the new node
	newNode->key = key;

	//  Check whether this is the first node
	if(_this->listSize == 0)
	{
		// Point head and tail node to first node
		_this->pHead = _this->pTail = newNode;
	}
	// We reached the max so squeeze out the front
	else if(_this->listSize == _this->maxListSize)
	{
		// Delete last node before adding new node
		List_deleteFirst(_this);
		// Otherwise we move the new node to back of the list
		_this->pTail->pNext = newNode;
		// Now point tail at new node
		_this->pTail = newNode;
	}
	else {
		// Otherwise we move the new node to back of the list
		_this->pTail->pNext = newNode;
		// Now point tail at new node
		_this->pTail = newNode;
	}

	// We increase list size
	_this->listSize++;

	/* Malloc past, re-enable the interrupts */
	__enable_irq();

	// Success
	return true;
}

/**
 * @brief   Remove the first (head node). This takes constant time
 * @param   *_this- pointer to the List_t
 */
void List_deleteFirst(List_t *_this)
{
	if(_this->listSize > 0)
	{
		Node_t *temp = _this->pHead;

		// We just want to free this memory

		if(_this->listSize > 1)
		{
			_this->pHead = _this->pHead->pNext;
		}
		// We need to make pointers null
		else {
			_this->pHead = NULL;
			_this->pTail = NULL;
		}

		// Decrease size
		_this->listSize--;

		// Free the temp (old head node) from memory, as well as its data
		free(temp->data);
		free(temp);
	}
}

/**
 * @brief   Remove the last node (tail node). This takes a variable and long
 *          amount of time
 * @param   *_this- pointer to the List_t
 */
void List_deleteLast(List_t *_this)
{
	if(_this->listSize > 0)
	{
		Node_t *temp = _this->pHead;
		Node_t *currNode = NULL;

		if(_this->listSize > 1)
		{
			// Traverse the list from head node onwards
			while(temp->pNext != NULL)
			{
				currNode = temp;
				temp = temp->pNext;
			}
			// Set the tail node to the current node - we know this is the second last node
			_this->pTail = currNode;

			// Now we are at the last node
			free(currNode->pNext->data);
			free(currNode->pNext);
			currNode->pNext = NULL; // Clear the pointer
		}
		// We need to make pointers null
		else {

			_this->pHead = NULL;
			_this->pTail = NULL;

			// Free the temp (old head node) from memory, as well as its data
			free(temp->data);
			free(temp);
		}

		// Decrease size
		_this->listSize--;
	}
}


/**
 * @brief   Remove the node number given(traverse the list). This takes a variable and long
 *          amount of time
 * @param   *_this- pointer to the List_t
 */
bool List_deleteItem(List_t *_this, uint8_t index)
{
    Node_t *temp = NULL;
    Node_t *curr = NULL;
    Node_t *prev = NULL;
    Node_t *del = NULL;
    uint8_t i = 0;

    // Start from beginning
    temp = curr = _this->pHead;

    if(_this->listSize > 0 && index < _this->listSize)
    {
        // traverse the list
        while(curr != NULL && i <= index)
        {
            prev = temp;        // Try and keep the previous ptr
            temp = curr;
            curr = curr->pNext;

            i++;
        }

        // Last node
        if(curr == NULL)
        {
            _this->pTail = prev;
        }
        // This is actually the first node
        if(i == 1)
        {
            _this->pHead = curr;
            if(_this->pHead != NULL)
                _this->pHead->pNext = curr->pNext;
        }

        // Index was found - return the data
        if(i == (index+1))
        {
//            del = curr;
            del = temp;

            // Move previous to the next next pointer
            if(prev != NULL)
                prev->pNext = temp->pNext;

            // Free the data
            free(del->data);
            free(del);

            // Decrease list size
            _this->listSize--;

            // Remove the tail dangling pointer
            if(_this->listSize == 0) _this->pTail = NULL;


            // The index was found
            return true;
        }
    }
    else {
        return false;
    }

    return false;
}

/**
 * @brief   Remove the node number given(traverse the list). This takes a variable and long
 *          amount of time
 * @param   *_this- pointer to the List_t
 */
bool List_deleteItemwithKey(List_t *_this, uint8_t key)
{
	int8_t key_pos;
	/* check if the MSG_INFORMATION is in the stash */
	key_pos = List_keyExists(_this, key, NULL);
	/* remove it from the stash if it is there */
	if (key_pos != -1)
	{
		//Remove the message from the stash
		List_deleteItem(_this, key_pos);
		return true;
	}

	return false;
}



/**
 * @brief   Remove the all the nodes from the list
 * @param   *_this- pointer to the List_t
 */
void List_empty(List_t *_this)
{
	while(_this->listSize > 0)
	{
		Node_t *temp = _this->pHead;

		// We just want to free this memory

		if(_this->listSize > 1)
		{
			_this->pHead = _this->pHead->pNext;
		}
		// We need to make pointers null
		else {
			_this->pHead = NULL;
			_this->pTail = NULL;
		}

		// Decrease size
		_this->listSize--;

		// Free the temp (old head node) from memory, as well as its data
		free(temp->data);
		free(temp);
	}
}


/**
 * @brief   Get the data of the last element of the list
 * @param   *_this - pointer to the List_t
 */
void List_getLast(const List_t *_this, void *data)
{
	Node_t *temp = NULL;

	if(_this->listSize > 0)
	{
		temp = _this->pTail;
		// Copy the data from the tail node
		memcpy(data, temp->data, _this->memorySize);
	}
	else {
		data = NULL;
	}
}

///**
// * @brief   Get the data of the last element of the list
// * @param   *_this - pointer to the List_t
// * @param   *data - the date to put in
// */
//bool List_getItem(List_t *_this, uint8_t index, void *data)
//{
//	Node_t *temp = NULL;
//	Node_t *curr = NULL;
//	uint8_t i = 0;
//
//	// Start from beginning
//	temp = curr = _this->pHead;
//
//	if(_this->listSize > 0 && index < _this->listSize)
//	{
//		// traverse the list
//		while(curr != NULL && i <= index)
//		{
//			temp = curr;
//			curr = curr->pNext;
//
//			i++;
//		}
//
////		if(curr == NULL) return false;
//
//		// Index was found - return the data
//		if(i == (index+1))
//		{
//			// Copy the data from the tail node
//			memcpy(data, temp->data, _this->memorySize);
//		}
//
//
//		// The index was found
//		return true;
//	}
//	else {
//		data = NULL;
//		return false;
//	}
//
//	return false;
//}


/**
 * @brief   Get the data of the last element of the list
 * @param   *_this - pointer to the List_t
 * @param   *data - the date to put in
 */
bool List_getItem(List_t *_this, uint8_t index, void *data)
{
    Node_t *temp = NULL;
    Node_t *curr = NULL;
    uint8_t i = 0;

    // Start from beginning
    temp = curr = _this->pHead;

    if(_this->listSize > 0 && index < _this->listSize)
    {
        // traverse the list
        while(curr != NULL && i <= index)
        {
            temp = curr;
            curr = curr->pNext;

            i++;
        }

//        if(curr == NULL) return false;
        if(temp == NULL) return false;

        // Index was found - return the data
        if(i == (index+1))
        {
            // Copy the data from the tail node
            memcpy(data, temp->data, _this->memorySize);
        }


        // The index was found
        return true;
    }
    else {
        data = NULL;
        return false;
    }

    return false;
}



/**
 * @brief	Check if the requested item exits. For now we will only search by pointer.
 * @retval	Return -1 if item wasn't found, otherwise return the index
 */
int8_t List_keyExists(List_t *_this, uint8_t key, void *data)
{
	Node_t *temp = NULL;
	Node_t *curr = NULL;
	uint8_t i = 0;

	// Start from beginning
	temp = curr = _this->pHead;

	if(_this->listSize > 0)
	{
		// traverse the list
		while(curr != NULL && i < _this->listSize)
		{
			temp = curr;
			curr = curr->pNext;

			if(temp->key == key)
			{
				// Check the 0 ptr
				if(data != NULL)
					// Copy the data from the tail node
					memcpy(data, temp->data, _this->memorySize);
				return i;
			}

			i++;
		}

		if(curr == NULL) {
			data = NULL;
			return -1;
		}


		// The index was found
		return true;
	}
	else {
		data = NULL;
		return -1;
	}

	data = NULL;
	return -1;
}

/**
 * @brief   Edit the specific item in the list
 * @param   *_this - pointer to the List_t
 * @param   *data - the date to put in
 */
bool List_editItem(List_t *_this, uint8_t index, void *data)
{
	Node_t *temp = NULL;
	Node_t *curr = NULL;
	uint8_t i = 0;

	// Start from beginning
	temp = curr = _this->pHead;

	if(_this->listSize > 0 && index < _this->listSize)
	{
		// traverse the list
		while(curr != NULL && i <= index)
		{
			temp = curr;
			curr = curr->pNext;

			i++;
		}

//		if(curr == NULL) return false;

		// Index was found - return the data
		if(i == (index+1))
		{
			// Copy the data from the tail node
			memcpy(temp->data, data, _this->memorySize);
		}


		// The index was found
		return true;
	}
	else {
		data = NULL;
		return false;
	}

	return false;
}


/**
 * @brief   Initialise the list. We need to know the size of data type of list
 * @param   *_this - pointer to the List_t
 * @retval  [uint8_t] size of this List
 */
uint8_t List_getSize(const List_t *_this)
{
	return _this->listSize;
}

/**
 * @brief   Initialise the list. We need to know the size of data type of list
 * @param   *_this - pointer to the List_t
 * @retval  true if list is empty; false otherwise
 */
bool List_isEmpty(const List_t *_this)
{
	return (_this->pHead == NULL);
}

