/******************************************************************************
 *                                                                            *
 *                                                                            *
 *                           	Copyright by                                  *
 *                                                                            *
 *                         	  Azoteq (Pty) Ltd                                *
 *                     	  Republic of South Africa                            *
 *                                                                            *
 *                     	   Tel: +27(0)21 863 0033                             *
 *                    	  E-mail: info@azoteq.com                             *
 *                                                                            *
 *============================================================================*
 * @file 	    AZQ_util.c						  	 				  	 	  *
 * @brief 	    Utilities for Azoteq Luminaire				  		  	  	  *
 * @author 		FP Stegmann - Azoteq PTY Ltd                             	  *
 * @version 	V1.0.0                                                    	  *
 * @date 	    23/08/2018                                                    *
 ******************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "AZQ_util.h"
#include <stdbool.h>
#include <string.h>
#include <stdio.h>

/* callback function */
void (*_exti_callbacks[16])(uint8_t);

static uint32_t tim7_residual = 0;

/**
  * @brief  HAL tick from systimer
  * @retval None
  */

uint32_t HAL_GetTick(void) {
    extern volatile uint32_t msTick;

#ifdef	TIM7_BASED_MSTICK
	uint32_t calc_value;

	calc_value = ((TIM7->CNT+tim7_residual)/500);

	return (msTick+calc_value);
#else
	return msTick;
#endif
}

/**
  * @brief  Check if an amount of time has passed
  * @retval None
  */

bool HAL_CheckTick(uint32_t timestamp, uint32_t compare_value, bool larger_check) {
    extern volatile uint32_t msTick;
    uint32_t calc_value = 0;

#ifdef	TIM7_BASED_MSTICK

	calc_value = ((TIM7->CNT+tim7_residual)/500);
#endif

    /* check that the timestamp was taken during the current msTick run cycle */
    if (timestamp <= (msTick+calc_value))
    {
    	/* larger check */
    	if (larger_check)
    	{
			if (((msTick+calc_value) - timestamp) >= compare_value)
			{
				return true;
			}
			else
			{
				return false;
			}
    	}
    	/* smaller check */
    	else
    	{
			if (((msTick+calc_value) - timestamp) < compare_value)
			{
				return true;
			}
			else
			{
				return false;
			}
    	}
    }
    /* the timestamp was taken in a previous msTick cycle */
    else
    {
    	/* larger check */
    	if (larger_check)
    	{
        	/* compensate for the msTick rollover */
        	if (((((uint32_t) ~1) - timestamp) + (msTick+calc_value)) >= compare_value)
        	{
        		return true;
        	}
        	else
        	{
        		return false;
        	}
    	}
    	/* smaller check */
    	else
    	{
        	/* compensate for the msTick rollover */
        	if (((((uint32_t) ~1) - timestamp) + (msTick+calc_value)) < compare_value)
        	{
        		return true;
        	}
        	else
        	{
        		return false;
        	}
    	}
    }

    /* the function did not generate a meaningful return */
	return false;
}

/**
  * @brief  Convert integer value to padded char string
  * @note
  * @param  integer value to be coverted
  * @retval char sting padded correctly
  */
void print_int(uint16_t value, char* int_char_buffer)
{

	uint8_t num_chars = 0;
	uint8_t i;

	/* Sanity check */
	if (value > 9999 || value < -999)
		return;


	/* Check how many characters will be printed */
	if (value / 1000 >= 1 || value / -100 >= 1) {
		num_chars = 4;
	}
	else if (value / 100 >= 1 || value / -10 >= 1) {
		num_chars = 3;
	}
	else if (value / 10 >= 1 || value / -1 >= 1) {
		num_chars = 2;
	}
	else {
		num_chars = 1;
	}

	/* Insert white space in front of printed value */
	for (i = 0; i < 4 - num_chars; i++) {
		int_char_buffer[i] = ' ';
	}

	sprintf(&int_char_buffer[4 - num_chars], "%d", value);
}


/**
  * @brief  Configures the timer as a time base.
  * @note   Peripheral configuration is minimal configuration from reset values.
  *         Thus, some useless LL unitary functions calls below are provided as
  *         commented examples - setting is default configuration from reset.
  * @param  None
  * @retval None
  */
void Configure_TIM7(void)
{
	uint32_t prescaler = 0;

  /* Enable the timer peripheral clock */
	LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_TIM7);

  /* Set counter mode */
  /* Reset value is LL_TIM_COUNTERMODE_UP */
  LL_TIM_SetCounterMode(TIM7, LL_TIM_COUNTERMODE_UP);

  /* Set the pre-scaler value to have TIM2 counter clock equal to 10 kHz      */
  /*
    In this example TIM2 input clock (TIM2CLK)  is set to APB1 clock (PCLK1),
    since APB1 prescaler is equal to 1.
      TIM2CLK = PCLK1
      PCLK1 = HCLK
      => TIM2CLK = HCLK = SystemCoreClock
    To get TIM2 counter clock at 10 KHz, the Prescaler is computed as following:
    Prescaler = (TIM2CLK / TIM2 counter clock) - 1
    Prescaler = (SystemCoreClock /10 KHz) - 1
  */
  prescaler = __LL_TIM_CALC_PSC(SystemCoreClock, 500000);
  LL_TIM_SetPrescaler(TIM7, prescaler);

  /* Set the auto-reload value to have an initial update event frequency of 1000 Hz */
  LL_TIM_DisableARRPreload(TIM7);
//  LL_TIM_SetAutoReload(TIM7, 65535);

  /* Enable the update interrupt */
//  LL_TIM_EnableIT_UPDATE(TIM7);

  /* Configure the NVIC to handle TIM7 update interrupt */
//  NVIC_SetPriority(TIM7_IRQn, 0x00);
//  NVIC_EnableIRQ(TIM7_IRQn);

  /* Enable counter */
  LL_TIM_EnableCounter(TIM7);
}

/**
 * @brief	update the msTick
 */
void AZQ_update_msTick(void)
{
	uint32_t values, temp = 0;
    extern volatile uint32_t msTick;

    temp = TIM7->CNT;
	LL_TIM_SetCounter(TIM7, 0);
	values = (temp+tim7_residual)/500;
	tim7_residual = (temp+tim7_residual)%500;

	msTick = msTick + values;
}



/**
 * @brief	Enable Selected GPIO port clock, null argument enables all defined GPIO ports clocks
 */
void Enable_GPIO_Clock(GPIO_TypeDef *gpiox)

{

	uint8_t gpio_index = RETURN_GPIO_ENUM(gpiox);


	switch(gpio_index)
	{
		case 0:
			LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA);
		break;

		case 1:
			LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOB);
		break;

		case 2:
			LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOC);
		break;

		case 3:
			LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOD);
		break;

		default:
			LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_ALL);
	}
}

/**
 * @brief	Enable selected I2C Clock.
 */

void Enable_I2C_Clock(I2C_TypeDef *I2Cx)
{
	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);
	switch(i2c_index)
	{
		case 0:
			LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_I2C1);
		break;

		case 1:
			LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_I2C2);
		break;
	}

}


/**
 * @brief	Enable and set priority of selected I2C peripheral
 */

void init_I2C_interrupt(I2C_TypeDef *I2Cx, uint32_t priority)
{

	uint8_t i2c_index = RETURN_I2C_ENUM(I2Cx);

	switch(i2c_index)
	{
		/* set priority and enable NVIC IRQ for I2C1 */
		case 0:
			  NVIC_SetPriority(I2C1_IRQn, priority);
			  NVIC_EnableIRQ(I2C1_IRQn);
		break;

		/* set priority and enable NVIC IRQ for I2C2 */
		case 1:
			  NVIC_SetPriority(I2C2_IRQn, priority);
			  NVIC_EnableIRQ(I2C2_IRQn);
		break;
	}

}

/**
 * @brief	Initialise the general purpose IOs for any IO
 * @param	Set PORT [GPIO_TypeDef *] GPIOA-GPIOx
 * @param	Set Pin Number [uint32_t] LL_GPIO_PIN_0 - LL_GPIO_PIN_X
 * @param	Set Pin Mode [uint32_t] LL_GPIO_MODE_INPUT, LL_GPIO_MODE_OUTPUT, LL_GPIO_MODE_ALTERNATE, LL_GPIO_MODE_ANALOG
 * @param	Set Pin Alternate Function [uint32_t] LL_GPIO_AF_1-LL_GPIO_AF_X
 * @param	Set pin output type [uint32_t] LL_GPIO_OUTPUT_PUSHPULL, LL_GPIO_OUTPUT_OPENDRAIN
 * @param   Set Pin Pull [uint32_t] LL_GPIO_PULL_NO, LL_GPIO_PULL_UP, LL_GPIO_PULL_DOWN
 * @param	Set Pin Speed [uint32_t] LL_GPIO_SPEED_FREQ_LOW, LL_GPIO_SPEED_FREQ_MEDIUM, LL_GPIO_SPEED_FREQ_HIGH, LL_GPIO_SPEED_FREQ_VERY_HIGH
 */

void gpio_init(GPIO_TypeDef *gpiox, uint32_t pinx, uint32_t mode, uint32_t af_function, uint32_t type, uint32_t pu_pd, uint32_t speed)
{

	LL_GPIO_SetPinMode(gpiox, pinx, mode);

	// Check whether alterante mode was selected, and which number pin to activate correct AF.
	if(mode == LL_GPIO_MODE_ALTERNATE)
	{
		if(pinx <= LL_GPIO_PIN_7)
		{
			LL_GPIO_SetAFPin_0_7(gpiox, pinx, af_function);
		} else if(pinx >= LL_GPIO_PIN_8)
		{
			LL_GPIO_SetAFPin_8_15(gpiox, pinx, af_function);
		}
	}

	// check whether necessary to set OD or PP for output pin.
	if(mode != LL_GPIO_MODE_INPUT)
	{
		LL_GPIO_SetPinOutputType(gpiox, pinx, type);
	}

	LL_GPIO_SetPinSpeed(gpiox, pinx, speed);
	LL_GPIO_SetPinPull(gpiox, pinx, pu_pd);
}


/*
 * @brief The function is used to configure external interrupts.
 * @param [GPIO_TypeDef] gpiox : Port of pin which is used for external interrupt (GPIOA - GPIOx)
 * @param [uint32_t] IR_Line: Set interrupt line (LL_EXTI_LINE_0 - LL_EXTI_LINE_15)
 * @param [uint8_t] IR_priority: Set priority of interrupt
 * @param [bool] rising: Enable rising trigger (True/false)
 * @param [bool] falling: Enable falling trigger (True/false)
 *
 */

void Configure_EXTI(GPIO_TypeDef *gpiox, uint32_t IR_Line, uint32_t IR_priority, bool rising, bool falling, void *_exti_callback)
{

	uint8_t gpio_index = RETURN_GPIO_ENUM(gpiox);
	uint8_t IR_Index = Get_IR_Index(IR_Line);
	uint32_t IR_Config_Line;
	uint32_t IR_EXTI_Line;


	switch(IR_Line)
	{
		case LL_GPIO_PIN_0:
			IR_EXTI_Line = LL_EXTI_LINE_0;
			IR_Config_Line = LL_EXTI_CONFIG_LINE0;

		break;

		case LL_GPIO_PIN_1:
			IR_EXTI_Line = LL_EXTI_LINE_1;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE1;

		break;

		case LL_GPIO_PIN_2:
			IR_EXTI_Line = LL_EXTI_LINE_2;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE2;

		break;

		case LL_GPIO_PIN_3:
			IR_EXTI_Line = LL_EXTI_LINE_3;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE3;

		break;

		case LL_GPIO_PIN_4:
			IR_EXTI_Line = LL_EXTI_LINE_4;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE4;

		break;

		case LL_GPIO_PIN_5:
			IR_EXTI_Line = LL_EXTI_LINE_5;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE5;

		break;

		case LL_GPIO_PIN_6:
			IR_EXTI_Line = LL_EXTI_LINE_6;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE6;

		break;

		case LL_GPIO_PIN_7:
			IR_EXTI_Line = LL_EXTI_LINE_7;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE7;

		break;

		case LL_GPIO_PIN_8:
			IR_EXTI_Line = LL_EXTI_LINE_8;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE8;

		break;

		case LL_GPIO_PIN_9:
			IR_EXTI_Line = LL_EXTI_LINE_9;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE9;

		break;

		case LL_GPIO_PIN_10:
			IR_EXTI_Line = LL_EXTI_LINE_10;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE10;

		break;

		case LL_GPIO_PIN_11:
			IR_EXTI_Line = LL_EXTI_LINE_11;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE11;

		break;

		case LL_GPIO_PIN_12:
			IR_EXTI_Line = LL_EXTI_LINE_12;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE12;

		break;

		case LL_GPIO_PIN_13:
			IR_EXTI_Line = LL_EXTI_LINE_13;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE13;

		break;

		case LL_GPIO_PIN_14:
			IR_EXTI_Line = LL_EXTI_LINE_14;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE14;

		break;

		case LL_GPIO_PIN_15:
			IR_EXTI_Line = LL_EXTI_LINE_15;
			IR_Config_Line =  LL_EXTI_CONFIG_LINE15;

		break;
	}

	 /* Check which GPIO Port is selected and Connect External Line to the GPIO*/
	switch(gpio_index)
	{
		case 0:
			  LL_EXTI_SetEXTISource(LL_EXTI_CONFIG_PORTA, IR_Config_Line);
		break;

		case 1:
			  LL_EXTI_SetEXTISource(LL_EXTI_CONFIG_PORTB, IR_Config_Line);
		break;

		case 2:
			  LL_EXTI_SetEXTISource(LL_EXTI_CONFIG_PORTC, IR_Config_Line);
		break;
	}

	// check whether rising trigger must be enabled.
	if(rising == true)
	{
		LL_EXTI_EnableRisingTrig_0_31(IR_EXTI_Line);
	}

	// check whether falling trigger must be enabled.
	if(falling == true)
	{
		LL_EXTI_EnableFallingTrig_0_31(IR_EXTI_Line);
	}


	  /*-4- Configure NVIC for selected EXTI Line */
	if(IR_EXTI_Line <= LL_EXTI_LINE_1)
	{
		  NVIC_SetPriority(EXTI0_1_IRQn, IR_priority);
		  NVIC_EnableIRQ(EXTI0_1_IRQn);

	} else if (IR_EXTI_Line <= LL_EXTI_LINE_3)
	{
		  NVIC_SetPriority(EXTI2_3_IRQn, IR_priority);
		  NVIC_EnableIRQ(EXTI2_3_IRQn);

	} else
	{
		  NVIC_SetPriority(EXTI4_15_IRQn, IR_priority);
		  NVIC_EnableIRQ(EXTI4_15_IRQn);
	}

	if(_exti_callback != NULL)
	{
		_exti_callbacks[IR_Index] = _exti_callback;
	}

}



void checkInterrupt_Line(uint32_t IR_Line)
{

	uint8_t exti_type = LL_EXTI_TRIGGER_NONE;
	uint8_t IR_Index = Get_IR_Index(IR_Line);

	// Check whether interrupt is enabled
	if(LL_EXTI_IsEnabledIT_0_31(IR_Line) != RESET)
	{
		// Check falling interrupt flag
		if(LL_EXTI_IsActiveFallingFlag_0_31(IR_Line) != RESET)
		{
			LL_EXTI_ClearFallingFlag_0_31(IR_Line);
			exti_type |= LL_EXTI_TRIGGER_FALLING;
		}

		// Check rising interrupt flag
		if(LL_EXTI_IsActiveRisingFlag_0_31(IR_Line) != RESET)
		{
			LL_EXTI_ClearRisingFlag_0_31(IR_Line);
			exti_type |= LL_EXTI_TRIGGER_RISING;
		}

		// Check whether interrupt occured
		if (exti_type != LL_EXTI_TRIGGER_NONE)
		{
			// call the assigned exti callback passing the interrupt type as argument
			if(_exti_callbacks[IR_Index] != NULL)
			{
				_exti_callbacks[IR_Index](exti_type);
			}
		}
	}
}

uint8_t Get_IR_Index(uint32_t IR_Line)
{
	uint8_t IR_Index = 0;
	while(IR_Line != 0)
	{
		IR_Line >>= 1;
		IR_Index++;
	}

	return (IR_Index-1);
}

/******************************************************************************/
/* STM32G0xx Peripheral Interrupt Handlers                                    */
/* Add here the Interrupt Handlers for the used peripherals.                  */
/* For the available peripheral interrupt handler names,                      */
/* please refer to the startup file (startup_stm32g0xx.s).                    */
/******************************************************************************/

/* USER CODE BEGIN 1 */
/**
 * @brief  This function handles external line 4_15 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI4_15_IRQHandler(void)
{
//	checkInterrupt_Line(LL_EXTI_LINE_4);
//	checkInterrupt_Line(LL_EXTI_LINE_5);
	checkInterrupt_Line(LL_EXTI_LINE_6);
//	checkInterrupt_Line(LL_EXTI_LINE_7);
//	checkInterrupt_Line(LL_EXTI_LINE_8);
//	checkInterrupt_Line(LL_EXTI_LINE_9);
	checkInterrupt_Line(LL_EXTI_LINE_10);
//	checkInterrupt_Line(LL_EXTI_LINE_11);
//	checkInterrupt_Line(LL_EXTI_LINE_12);
//	checkInterrupt_Line(LL_EXTI_LINE_13);
//	checkInterrupt_Line(LL_EXTI_LINE_14);
//	checkInterrupt_Line(LL_EXTI_LINE_15);

}


/**
 * @brief  This function handles external line 4_15 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI2_3_IRQHandler(void)
{
	checkInterrupt_Line(LL_EXTI_LINE_2);
	checkInterrupt_Line(LL_EXTI_LINE_3);
}

/**
 * @brief  This function handles external line 4_15 interrupt request.
 * @param  None
 * @retval None
 */
void EXTI0_1_IRQHandler(void)
{
	checkInterrupt_Line(LL_EXTI_LINE_0);
	checkInterrupt_Line(LL_EXTI_LINE_1);
}

